/*    ILFIVE lisp objects library -- stub functions for Ada-95 bindings
 *
 *    Copyright (c) 1991--1998 Alexandru Dan Corlan M.D.
 * 
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU Library General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    In short, you can use and modify this program. You can
 *    distribute the program with or without your modifications if you want,
 *    but you must not require licence fees and must provide the
 *    full source code with the distribution. If you make an application program
 *    which just calls functions from this library you may distribute that
 *    application program without source code, but also as an linkable object file, 
 *    and must distribute also this library with source code.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU Library General Public License for more details.
 *
 *    You should have received a copy of the GNU Library General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

\FileTitle{ % format: <short title> --- <long title>
ILIADA --- interface between the ILFIVE library and the GNU Ada compiler
}
\CopyrightNotice{ % format: Copyright \copyright <year(s)> <who>
Copyright \copyright 1994 Alexandru Dan CORLAN
}
\FileAuthor{ % the full name of the author
Alexandru Dan CORLAN
}
\RevisionHistory{ % write freely the revision history here

}
\vskip 1in

\begin{code}
--*/

#include <stdio.h>
#include <shpell5.h>

errhandle()

{
  fprintf(stderr, "\n\nERRHANDLE EXITED THE PROGRAM\n\n");
}

/* conses */

cell iliada_car(cell c)

{
  return car(c);
}


cell iliada_cdr(cell c)

{
  return cdr(c);
}


cell iliada_caar(cell c)

{
  return car(car(c));
}


cell iliada_cadr(cell c)

{
  return car(cdr(c));
}


cell iliada_cdar(cell c)

{
  return cdr(car(c));
}


cell iliada_cddr(cell c)

{
  return cdr(cdr(c));
}


cell iliada_caaar(cell c)

{
  return car(car(car(c)));
}


cell iliada_caadr(cell c)

{
  return car(car(cdr(c)));
}


cell iliada_cadar(cell c)

{
  return car(cdr(car(c)));
}


cell iliada_caddr(cell c)

{
  return car(cdr(cdr(c)));
}



cell iliada_cdaar(cell c)

{
  return cdr(car(car(c)));
}


cell iliada_cdadr(cell c)

{
  return cdr(car(cdr(c)));
}


cell iliada_cddar(cell c)

{
  return cdr(cdr(car(c)));
}


cell iliada_cdddr(cell c)

{
  return cdr(cdr(cdr(c)));
}



short iliada_listp(cell c)

{
  if(c==NIL || is_cons(c))
    return 1;
  else
    return 0;
}



short iliada_consp(cell c)

{
  if(is_cons(c))
    return 1;
  else
    return 0;
}

/* symbols */

char *symbol_name(cell c)

{
  return symname(c);
}

short iliada_symbolp(cell c)

{
  if(is_symbol(c))
    return 1;
  else 
    return 0;
}

/* strings */


short iliada_stringp(cell c)

{
  if(is_string(c))
    return 1;
  else 
    return 0;
}

char *iliada_cstringof(cell c)

{
  return string(c);
}

cell iliada_make_int(long v)

{
  cell r;

  r = lmake(BINARY);
  binary(r) = v;
  return r;
}


cell iliada_make_float(float v)

{
  cell r;

  r = lmake(FLOAT);
  value(r) = v;
  return r;
}

float iliada_float_of(cell c)

{
  return value(c);
}

long iliada_int_of(cell c)

{
  return binary(c);
}


short iliada_integerp(cell c)

{
  if(is_binary(c))
    return 1;
  else 
    return 0;
}


short iliada_floatp(cell c)

{
  if(is_float(c))
    return 1;
  else 
    return 0;
}



short iliada_numberp(cell c)

{
  if(is_float(c) || is_binary(c))
    return 1;
  else 
    return 0;
}

cell iliada_fopen(char *fn, int mode)

{
  cell rv;

  switch(mode) {
  case 0:
    rv = fileopen(fn, INPUT, CHAR_FILE);
    break;
  case 1:
    rv = fileopen(fn, OUTPUT, CHAR_FILE);
    break;
  case 2:
    rv = fileopen(fn, INPUT_OUTPUT, CHAR_FILE);
    break;
  default:
    rv = fileopen(fn, INPUT, CHAR_FILE);  /* !!! VERY BAD SOLUTION */
/*    printf("In default\n"); */
    break;
  }
  return rv;
}

extern cell csout;
extern cell csin;
extern cell cserr;
extern cell cskbd;

cell iliada_the_csout()

{
  return csout;
}

cell iliada_the_csin()

{
  return csin;
}

cell iliada_the_cserr()

{
  return cserr;
}

cell iliada_the_cskbd()

{
  return cskbd;
}

void iliada_close(cell c)

{
  fclose(disconnect(c));
}

short iliada_streamp(cell c)

{
  if(is_stream(c))
    return 1;
  else
    return 0;
}

cell the_nil()

{
  return NIL;
}

/*--
\end{code}

% MOVE THIS COMMENT TO THE END OF THE FILE
\endoffile

--*/

