/*    ILFIVE lisp objects library -- character names
 *
 *    Copyright (c) 1991--1998 Alexandru Dan Corlan M.D.
 * 
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU Library General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    In short, you can use and modify this program. You can
 *    distribute the program with or without your modifications if you want,
 *    but you must not require licence fees and must provide the
 *    full source code with the distribution. If you make an application program
 *    which just calls functions from this library you may distribute that
 *    application program without source code, but also as an linkable object file, 
 *    and must distribute also this library with source code.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU Library General Public License for more details.
 *
 *    You should have received a copy of the GNU Library General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

\begin{abstract}
This heading lists the codes of keys as returned by 
the ilkbin() function; changing here the meaning but not
the name and recompiling modifies both the function of
the ilkbin function and that of the user modules.
\end{abstract}

 */

#define KEY_BREAK 27
#define KEY_DOWN ('X'-'A'+1)
#define KEY_UP ('E'-'A'+1)
#define KEY_LEFT ('S'-'A'+1)
#define KEY_RIGHT ('D'-'A'+1)
#define KEY_HOME ('W'-'A'+1)
#define KEY_BACKSPACE 8
#define KEY_F0 128
#define KEY_F(n) (KEY_F0 + n)
#define KEY_DL ('Y'-'A'+1)
#define KEY_IL ('N'-'A'+1)
#define KEY_DC ('G'-'A'+1)
#define KEY_IC ('V'-'A'+1)
#define KEY_EIC ('V'-'A'+1)
#define KEY_CLEAR ('O'-'A'+1)
#define KEY_EOS ('B'-'A'+1)
#define KEY_EOL ('J'-'A'+1)
#define KEY_SF crtl(Z)
#define KEY_SR ('W'-'A'+1)
#define KEY_NPAGE ('C'-'A'+1)
#define KEY_PPAGE ('R'-'A'+1)
#define KEY_STAB ('K'-'A'+1)
#define KEY_CTAB ('L'-'A'+1)
#define KEY_ENTER ('M'-'A'+1)
#define KEY_SRESET 0
#define KEY_RESET 0
#define KEY_PRINT ('P'-'A'+1)
#define KEY_LL ('Z'-'A'+1)
#define KEY_A1 ('W'-'A'+1)
#define KEY_A3 ('R'-'A'+1)
#define KEY_B2 ('Q'-'A'+1)
#define KEY_C1 ('Z'-'A'+1)
#define KEY_C3 ('C'-'A'+1)


/*.pa additional keys*/

/* Aditional keys provided in CIFER */
/* Usage of these key is reserved for CID system applications only
  as they make such programs non-portable */

#define HOT_KEY0 192
#define HOT_KEY(n) (HOT_KEY0 + n)

/* i.e. ctrl & KEY */

#define PAD_A1 228
#define PAD_A2 229
#define PAD_A3 230
#define PAD_B1 231
#define PAD_B2 232
#define PAD_B3 233
#define PAD_C1 234
#define PAD_C2 235
#define PAD_C3 236

#define SPAD_A1 237
#define SPAD_A2 238
#define SPAD_A3 239
#define SPAD_B1 240
#define SPAD_B2 241
#define SPAD_B3 242
#define SPAD_C1 243
#define SPAD_C2 244
#define SPAD_C3 245

#define CPAD_A1 246
#define CPAD_A2 247
#define CPAD_A3 248
#define CPAD_B1 249
#define CPAD_B2 250
#define CPAD_B3 251
#define CPAD_C1 252
#define CPAD_C2 253
#define CPAD_C3 254

/* sinonims for keys to be used with IBM-PC */

#define KEY_PGUP KEY_PPAGE
#define KEY_PGDN KEY_NPAGE
#define KEY_END KEY_EOS
#define KEY_INS KEY_EIC
#define KEY_DEL KEY_DC

#define KEY_CTRL_LEFT PAD_B1
#define KEY_CTRL_UP PAD_A2
#define KEY_CTRL_DOWN PAD_C2
#define KEY_CTRL_RIGHT PAD_B3
#define KEY_CTRL_HOME PAD_A1
#define KEY_CTRL_PGUP PAD_A3
#define KEY_CTRL_PGDN PAD_C3
#define KEY_CTRL_END PAD_C1
#define KEY_CTRL_DEL PAD_B2
#define KEY_CTRL_INS CPAD_A1

#define KEY_ALT_LEFT SPAD_B1
#define KEY_ALT_UP SPAD_A2
#define KEY_ALT_DOWN SPAD_C2
#define KEY_ALT_RIGHT SPAD_B3
#define KEY_ALT_HOME SPAD_A1
#define KEY_ALT_PGUP SPAD_A3
#define KEY_ALT_PGDN SPAD_C3
#define KEY_ALT_END SPAD_C1
#define KEY_ALT_DEL SPAD_B2
#define KEY_ALT_INS CPAD_A2

#define KEY_ALT_BACKSPACE CPAD_A3
#define KEY_ALT_ENTER CPAD_B1
#define KEY_CTRL_BACKSPACE 127
#define KEY_CTRL_ENTER 10

#define KEY_F1 KEY_F(1)
#define KEY_F2 KEY_F(2)
#define KEY_F3 KEY_F(3)
#define KEY_F4 KEY_F(4)
#define KEY_F5 KEY_F(5)
#define KEY_F6 KEY_F(6)
#define KEY_F7 KEY_F(7)
#define KEY_F8 KEY_F(8)
#define KEY_F9 KEY_F(9)
#define KEY_F10 KEY_F(10)

#define KEY_CTRL_F1 KEY_F(11)
#define KEY_CTRL_F2 KEY_F(12)
#define KEY_CTRL_F3 KEY_F(13)
#define KEY_CTRL_F4 KEY_F(14)
#define KEY_CTRL_F5 KEY_F(15)
#define KEY_CTRL_F6 KEY_F(16)
#define KEY_CTRL_F7 KEY_F(17)
#define KEY_CTRL_F8 KEY_F(18)
#define KEY_CTRL_F9 KEY_F(19)
#define KEY_CTRL_F10 KEY_F(20)

#define KEY_SHIFT_F1 KEY_F(21)
#define KEY_SHIFT_F2 KEY_F(22)
#define KEY_SHIFT_F3 KEY_F(23)
#define KEY_SHIFT_F4 KEY_F(24)
#define KEY_SHIFT_F5 KEY_F(25)
#define KEY_SHIFT_F6 KEY_F(26)
#define KEY_SHIFT_F7 KEY_F(27)
#define KEY_SHIFT_F8 KEY_F(28)
#define KEY_SHIFT_F9 KEY_F(29)
#define KEY_SHIFT_F10 KEY_F(30)

#define KEY_ALT_F1 KEY_F(31)
#define KEY_ALT_F2 KEY_F(32)
#define KEY_ALT_F3 KEY_F(33)
#define KEY_ALT_F4 KEY_F(34)
#define KEY_ALT_F5 KEY_F(35)
#define KEY_ALT_F6 KEY_F(36)
#define KEY_ALT_F7 KEY_F(37)
#define KEY_ALT_F8 KEY_F(38)
#define KEY_ALT_F9 KEY_F(39)
#define KEY_ALT_F10 KEY_F(40)

#define KEY_F11 KEY_F(41)
#define KEY_F12 KEY_F(42)
#define KEY_CTRL_F11 KEY_F(43)
#define KEY_CTRL_F12 KEY_F(44)
#define KEY_SHIFT_F11 KEY_F(45)
#define KEY_SHIFT_F12 KEY_F(46)


#define KEY_ALT_A HOT_KEY(11)
#define KEY_ALT_B HOT_KEY(12)
#define KEY_ALT_C HOT_KEY(13)
#define KEY_ALT_D HOT_KEY(14)
#define KEY_ALT_E HOT_KEY(15)
#define KEY_ALT_F HOT_KEY(16)
#define KEY_ALT_G HOT_KEY(17)
#define KEY_ALT_H HOT_KEY(18)
#define KEY_ALT_I HOT_KEY(19)
#define KEY_ALT_J HOT_KEY(20)
#define KEY_ALT_K HOT_KEY(21)
#define KEY_ALT_L HOT_KEY(22)
#define KEY_ALT_M HOT_KEY(23)
#define KEY_ALT_N HOT_KEY(24)
#define KEY_ALT_O HOT_KEY(25)
#define KEY_ALT_P HOT_KEY(26)
#define KEY_ALT_Q HOT_KEY(27)
#define KEY_ALT_R HOT_KEY(28)
#define KEY_ALT_S HOT_KEY(29)
#define KEY_ALT_T HOT_KEY(30)
#define KEY_ALT_U HOT_KEY(31)
#define KEY_ALT_V HOT_KEY(32)
#define KEY_ALT_W HOT_KEY(33)
#define KEY_ALT_X HOT_KEY(34)
#define KEY_ALT_Y HOT_KEY(35)
#define KEY_ALT_Z HOT_KEY(36)

#define KEY_ALT_1 HOT_KEY(1)
#define KEY_ALT_2 HOT_KEY(2)
#define KEY_ALT_3 HOT_KEY(3)
#define KEY_ALT_4 HOT_KEY(4)
#define KEY_ALT_5 HOT_KEY(5)
#define KEY_ALT_6 HOT_KEY(6)
#define KEY_ALT_7 HOT_KEY(7)
#define KEY_ALT_8 HOT_KEY(8)
#define KEY_ALT_9 HOT_KEY(9)
#define KEY_ALT_0 HOT_KEY(10)

#define KEY_ALT_MINUS       KEY_F(47)
#define KEY_ALT_EQUAL       KEY_F(48)
#define KEY_ALT_OPENS       KEY_F(49)
#define KEY_ALT_CLOSES      KEY_F(50)
#define KEY_ALT_SEMICOLON   KEY_F(51)
#define KEY_ALT_QUOTE       KEY_F(52)
#define KEY_ALT_BACKQUOTE   KEY_F(53)
#define KEY_ALT_COMMA       KEY_F(54)
#define KEY_ALT_PERIOD      KEY_F(55)
#define KEY_ALT_SLASH       KEY_F(56)
#define KEY_ALT_BACKSLASH   KEY_F(57)

/* End of the chrnames.h file */


