/*    ILFIVE lisp objects library -- characters
 *
 *    Copyright (c) 1991--1998 Alexandru Dan Corlan M.D.
 * 
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU Library General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    In short, you can use and modify this program. You can
 *    distribute the program with or without your modifications if you want,
 *    but you must not require licence fees and must provide the
 *    full source code with the distribution. If you make an application program
 *    which just calls functions from this library you may distribute that
 *    application program without source code, but also as an linkable object file, 
 *    and must distribute also this library with source code.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU Library General Public License for more details.
 *
 *    You should have received a copy of the GNU Library General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

\title{ILFOUR --- character management package (il5char)}
\author{Alexandru Dan CORLAN}
\date{6th of July 1991}

\begin{abstract}
This file implements characters. The most important features of
ILFOUR characters are the facts that they have names and that
their set can be extended to multibyte characters to cover
any lamguage.
\end{abstract}


 */

#include <malloc.h>
#include <stdio.h>

#include <shpell5.h>
#include <chrnames.h>

cell keyname[256];  /* this number of keys is fixed */

/* \section{Finding a character given name --- a symbol} */

int sym2char(cell s)

  {
  int i, r;

  if(symname(s)[1]==0)
    return symname(s)[0];
  for(i=0; i<256; i++)
    if(keyname[i]==s)
      return i;
  slex1s("Character name %s does not exist.", symname(s));
  }


cell char2sym(int c)

  {
  static char symn[3];

  if(c<128 && c>=32)
    {
    symn[0] = c;
    symn[1] = 0;
    return esym(symn);   /* WARNING: this does not work with multiple packages! */
    }
  return keyname[c];
  }

char *char2str(int c)

  {
  cell r;

  r = char2sym(c);
  if(r==NIL)
    return (char *)NULL;
  return symname(r);
  }

il5char()

  {
   int i;

   for(i=0; i<256; i++)
     keyname[i] = NIL;

	keyname[KEY_BREAK ] = esym("BREAK");
	keyname[KEY_DOWN ] = esym("DOWN");
	keyname[KEY_UP ] = esym("UP");
	keyname[KEY_LEFT ] = esym("LEFT");
	keyname[KEY_RIGHT ] = esym("RIGHT");
	keyname[KEY_HOME ] = esym("HOME");
	keyname[KEY_BACKSPACE ] = esym("BACKSPACE");
	keyname[KEY_ENTER ] = esym("ENTER");

	keyname[KEY_PGUP ] = esym("PGUP");
	keyname[KEY_PGDN ] = esym("PGDN");
	keyname[KEY_END ] = esym("END");
	keyname[KEY_INS ] = esym("INS");
	keyname[KEY_DEL ] = esym("DEL");

	keyname[KEY_CTRL_LEFT ] = esym("CTRL+LEFT");
	keyname[KEY_CTRL_UP ] = esym("CTRL+UP");
	keyname[KEY_CTRL_DOWN ] = esym("CTRL+DOWN");
	keyname[KEY_CTRL_RIGHT ] = esym("CTRL+RIGHT");
	keyname[KEY_CTRL_HOME ] = esym("CTRL+HOME");
	keyname[KEY_CTRL_PGUP ] = esym("CTRL+PGUP");
	keyname[KEY_CTRL_PGDN ] = esym("CTRL+PGDN");
	keyname[KEY_CTRL_END ] = esym("CTRL+END");
	keyname[KEY_CTRL_DEL ] = esym("CTRL+DEL");
	keyname[KEY_CTRL_INS ] = esym("CTRL+INS");

	keyname[KEY_ALT_LEFT ] = esym("ALT+LEFT");
	keyname[KEY_ALT_UP ] = esym("ALT+UP");
	keyname[KEY_ALT_DOWN ] = esym("ALT+DOWN");
	keyname[KEY_ALT_RIGHT ] = esym("ALT+RIGHT");
	keyname[KEY_ALT_HOME ] = esym("ALT+HOME");
	keyname[KEY_ALT_PGUP ] = esym("ALT+PGUP");
	keyname[KEY_ALT_PGDN ] = esym("ALT+PGDN");
	keyname[KEY_ALT_END ] = esym("ALT+END");
	keyname[KEY_ALT_DEL ] = esym("ALT+DEL");
	keyname[KEY_ALT_INS ] = esym("ALT+INS");

	keyname[KEY_ALT_BACKSPACE ] = esym("ALT+BACKSPACE");
	keyname[KEY_ALT_ENTER ] = esym("ALT+ENTER");
	keyname[KEY_CTRL_BACKSPACE ] = esym("CTRL+BACKSPACE");
	keyname[KEY_CTRL_ENTER ] = esym("CTRL+ENTER");

	keyname[KEY_F1 ] = esym("F1");
	keyname[KEY_F2 ] = esym("F2");
	keyname[KEY_F3 ] = esym("F3");
	keyname[KEY_F4 ] = esym("F4");
	keyname[KEY_F5 ] = esym("F5");
	keyname[KEY_F6 ] = esym("F6");
	keyname[KEY_F7 ] = esym("F7");
	keyname[KEY_F8 ] = esym("F8");
	keyname[KEY_F9 ] = esym("F9");
	keyname[KEY_F10 ] = esym("F10");

	keyname[KEY_CTRL_F1 ] = esym("CTRL+F1");
	keyname[KEY_CTRL_F2 ] = esym("CTRL+F2");
	keyname[KEY_CTRL_F3 ] = esym("CTRL+F3");
	keyname[KEY_CTRL_F4 ] = esym("CTRL+F4");
	keyname[KEY_CTRL_F5 ] = esym("CTRL+F5");
	keyname[KEY_CTRL_F6 ] = esym("CTRL+F6");
	keyname[KEY_CTRL_F7 ] = esym("CTRL+F7");
	keyname[KEY_CTRL_F8 ] = esym("CTRL+F8");
	keyname[KEY_CTRL_F9 ] = esym("CTRL+F9");
	keyname[KEY_CTRL_F10 ] = esym("CTRL+F10");

	keyname[KEY_SHIFT_F1 ] = esym("SHIFT+F1");
	keyname[KEY_SHIFT_F2 ] = esym("SHIFT+F2");
	keyname[KEY_SHIFT_F3 ] = esym("SHIFT+F3");
	keyname[KEY_SHIFT_F4 ] = esym("SHIFT+F4");
	keyname[KEY_SHIFT_F5 ] = esym("SHIFT+F5");
	keyname[KEY_SHIFT_F6 ] = esym("SHIFT+F6");
	keyname[KEY_SHIFT_F7 ] = esym("SHIFT+F7");
	keyname[KEY_SHIFT_F8 ] = esym("SHIFT+F8");
	keyname[KEY_SHIFT_F9 ] = esym("SHIFT+F9");
	keyname[KEY_SHIFT_F10 ] = esym("SHIFT+F10");

	keyname[KEY_ALT_F1 ] = esym("ALT+F1");
	keyname[KEY_ALT_F2 ] = esym("ALT+F2");
	keyname[KEY_ALT_F3 ] = esym("ALT+F3");
	keyname[KEY_ALT_F4 ] = esym("ALT+F4");
	keyname[KEY_ALT_F5 ] = esym("ALT+F5");
	keyname[KEY_ALT_F6 ] = esym("ALT+F6");
	keyname[KEY_ALT_F7 ] = esym("ALT+F7");
	keyname[KEY_ALT_F8 ] = esym("ALT+F8");
	keyname[KEY_ALT_F9 ] = esym("ALT+F9");
	keyname[KEY_ALT_F10 ] = esym("ALT+F10");

	keyname[KEY_F11 ] = esym("F11");
	keyname[KEY_F12 ] = esym("F12");
	keyname[KEY_CTRL_F11 ] = esym("CTRL+F11");
	keyname[KEY_CTRL_F12 ] = esym("CTRL+F12");
	keyname[KEY_SHIFT_F11 ] = esym("SHIFT+F11");
	keyname[KEY_SHIFT_F12 ] = esym("SHIFT+F12");


	keyname[KEY_ALT_A ] = esym("ALT+A");
	keyname[KEY_ALT_B ] = esym("ALT+B");
	keyname[KEY_ALT_C ] = esym("ALT+C");
	keyname[KEY_ALT_D ] = esym("ALT+D");
	keyname[KEY_ALT_E ] = esym("ALT+E");
	keyname[KEY_ALT_F ] = esym("ALT+F");
	keyname[KEY_ALT_G ] = esym("ALT+G");
	keyname[KEY_ALT_H ] = esym("ALT+H");
	keyname[KEY_ALT_I ] = esym("ALT+I");
	keyname[KEY_ALT_J ] = esym("ALT+J");
	keyname[KEY_ALT_K ] = esym("ALT+K");
	keyname[KEY_ALT_L ] = esym("ALT+L");
	keyname[KEY_ALT_M ] = esym("ALT+M");
	keyname[KEY_ALT_N ] = esym("ALT+N");
	keyname[KEY_ALT_O ] = esym("ALT+O");
	keyname[KEY_ALT_P ] = esym("ALT+P");
	keyname[KEY_ALT_Q ] = esym("ALT+Q");
	keyname[KEY_ALT_R ] = esym("ALT+R");
	keyname[KEY_ALT_S ] = esym("ALT+S");
	keyname[KEY_ALT_T ] = esym("ALT+T");
	keyname[KEY_ALT_U ] = esym("ALT+U");
	keyname[KEY_ALT_V ] = esym("ALT+V");
	keyname[KEY_ALT_W ] = esym("ALT+W");
	keyname[KEY_ALT_X ] = esym("ALT+X");
	keyname[KEY_ALT_Y ] = esym("ALT+Y");
	keyname[KEY_ALT_Z ] = esym("ALT+Z");

	keyname[KEY_ALT_1 ] = esym("ALT+1");
	keyname[KEY_ALT_2 ] = esym("ALT+2");
	keyname[KEY_ALT_3 ] = esym("ALT+3");
	keyname[KEY_ALT_4 ] = esym("ALT+4");
	keyname[KEY_ALT_5 ] = esym("ALT+5");
	keyname[KEY_ALT_6 ] = esym("ALT+6");
	keyname[KEY_ALT_7 ] = esym("ALT+7");
	keyname[KEY_ALT_8 ] = esym("ALT+8");
	keyname[KEY_ALT_9 ] = esym("ALT+9");
	keyname[KEY_ALT_0 ] = esym("ALT+0");

	keyname[KEY_ALT_MINUS ] = esym("ALT+MINUS");
	keyname[KEY_ALT_EQUAL ] = esym("ALT+EQUAL");
	keyname[KEY_ALT_OPENS ] = esym("ALT+OPENS");
	keyname[KEY_ALT_CLOSES ] = esym("ALT+CLOSES");
	keyname[KEY_ALT_SEMICOLON ] = esym("ALT+SEMICOLON");
	keyname[KEY_ALT_QUOTE ] = esym("ALT+QUOTE");
	keyname[KEY_ALT_BACKQUOTE ] = esym("ALT+BACKQUOTE");
	keyname[KEY_ALT_COMMA ] = esym("ALT+COMMA");
	keyname[KEY_ALT_PERIOD ] = esym("ALT+PERIOD");
	keyname[KEY_ALT_SLASH ] = esym("ALT+SLASH");
	keyname[KEY_ALT_BACKSLASH ] = esym("ALT+BACKSLASH");

   /* keys required by the CommonLisp standard */

   keyname[0] = esym("Null");
   keyname['\n'] = esym("Newline");
   keyname[12] = esym("FormFeed");
  }

/* End of the character management package */


