/*    ILFIVE lisp objects library -- functions for equality
 *
 *    Copyright (c) 1991--1998 Alexandru Dan Corlan M.D.
 * 
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU Library General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    In short, you can use and modify this program. You can
 *    distribute the program with or without your modifications if you want,
 *    but you must not require licence fees and must provide the
 *    full source code with the distribution. If you make an application program
 *    which just calls functions from this library you may distribute that
 *    application program without source code, but also as an linkable object file, 
 *    and must distribute also this library with source code.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU Library General Public License for more details.
 *
 *    You should have received a copy of the GNU Library General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <stdio.h>
#include <string.h>

#include <shpell5.h>

cell eq_;
cell eql_;
cell equal_;

int eql(cell x, cell y)

  {
  int i;

  if(id(x)!=id(y)) return 0;
  if(eq(x,y)) return 1;
  switch(id(x))
    {
    case SYMBOL:
    case PACKAGE:
    case LIST:
    case HASHTABLE:   /* this very case means breaking the standard */
      return 0;  /* as they are not eq */
    case BINARY:
      return binary(x)==binary(y);
    case STRING:
    case PURESTRING:
      return !strcmp(string(x), string(y));
    case FLOAT:
      return value(x)==value(y);
    case COMPLEX:
      return imag(x)==imag(y) && real(x)==real(y);
    case VECTOR:
      if (vecsize(x) != vecsize(y))
       return (0);
      for (i=0;i<vecsize(x);i++)
       if (!equal(vnth(x,i),vnth(y,i)))
         return (0);
      return (1);

    default:
      slex("Objects of unknown type were given argument to eql.");
    }
  return 0;  /* never reached */
  }

int equal(cell x, cell y)

  {
  if (!is_cons(x) && !is_cons(y)) return eql(x,y);
  if (is_cons(x) && is_cons(y))
    return equal(car(x),car(y)) && equal(cdr(x),cdr(y));
  return 0;  /* not even their ids are identical */
  }

int is_eq(cell mod, cell x, cell y)

  {
  if(mod==eq_)
    if(x==y)
      return 1;
    else
      return 0;
  if(mod==eql_) return eql(x,y);
  if(mod==equal_) return equal(x,y);
  slex("is_eq must be called with eq, eql or equal modes only.");
  }

il5eq()  /* this small package needs initialisation */

  {
  eq_ = esym("eq");
  eql_ = esym("eql");
  equal_ = esym("equal");
  }

 
