/*
 *  Il - V4.0. Open Common Lisp implementation
 *  for UNIX-like OSs based microcomputers
 *  with word of 16 or (better) 32 bits.
 *
 *  This file is part of the "runtime" of the system
 *  used by both the interpreter and the compiled
 *  code. Part of the program is rewritten after
 *  the MAGICK system, a runtime for Il V3.0
 *  which Il was never written completelly.
 *
 *  This package implements simple list processing
 *  machinery for which resemble or not the CommonLisp
 *  one.
 *
 *  written by Alexandru Dan Corlan
 *
 * Created 12th of March 1991.
 */

#include <stdio.h>

#include <shpell5.h>

cell dreverse(cell list)   /* destructivelly reverse a list */

  {
  cell x;
  cell y;

  y = NIL;
  while(list !=NIL )
    {
    x = list;
    y = cons(car(list), y);
    list = cdr(list);
    ungcell(x);
    }
  return y;
  }

cell copy(cell list) /* written such that the copied item can be ungstruct-ed
                      * without any harm; i.e. for each type a copy
                      * is given except for strings
                      */

  {
  cell rv;

  if(is_binary(list))
    return make(BINARY, binary(list));
  if(is_string(list))
    return strbuild(string(list));
  if(is_float(list))
    {
    rv = lmake(FLOAT);
    value(rv) = value(list);
    return rv;
    }
  /* more of them to be added here */

  if (!is_list(list) || list==NIL)
    return list;
  return cons(copy(car(list)), copy(cdr(list)));
  }

cell toplout(cell list)

  {
  cell x;

  while(list!=NIL)
    {
    x = list;
    list = cdr(list);
    ungcell(x);
    }
  return NIL;
  }

cell topasout(cell c)

  {
  cell x;

  /* c being an association list it throws it in the darkness
   * of the freelist
   */

  while(c!=NIL)
    {
    ungcell(car(c));
    x = c;
    c = cdr(c);
    ungcell(x);
    }
  }

cell assoc(cell sym, cell alist)

  {
  while(alist!=NIL)
    {
    if (sym==caar(alist)) return cdar(alist);
    alist = cdr(alist);
    }
  return NIL;
  }

cell topcopy(cell list)

  {
  if(list==NIL)
    return NIL;
  return cons(car(list), topcopy(cdr(list)));
  }

cell passoc(cell sym, cell palist)

  {
  while(palist!=NIL)
    {
    if (sym==car(palist)) return cadr(palist);
    palist = cddr(palist);
    }
  return NIL;
  }

int length(cell lis)

  {
  register rv;

  rv = 0;
  while(lis!=NIL)
    {
    rv++;
    lis = cdr(lis);
    }
  return rv;
  }

cell nth(cell lis, int n)

  {
  while(n>0 && lis!=NIL && is_cons(lis))
    {
    n--;
    lis = cdr(lis);
    }
  if(n>0) return NIL;
  if(is_cons(lis)) return car(lis);
  return NIL;  /* little ambiguity */
  }

cell nthcdr(cell lis, int n)

  {
  while(n>0 && lis!=NIL && is_cons(lis))
    {
    n--;
    lis = cdr(lis);
    }
  if(n>0) return NIL;
  if(is_cons(lis)) return lis;
  return NIL;  /* small ambiguity */
  }

/* end of il5lisp */



 
