/*    ILFIVE lisp objects library -- numbers (and vectors)
 *
 *    Copyright (c) 1991--1998 Alexandru Dan Corlan M.D.
 * 
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU Library General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    In short, you can use and modify this program. You can
 *    distribute the program with or without your modifications if you want,
 *    but you must not require licence fees and must provide the
 *    full source code with the distribution. If you make an application program
 *    which just calls functions from this library you may distribute that
 *    application program without source code, but also as an linkable object file, 
 *    and must distribute also this library with source code.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU Library General Public License for more details.
 *
 *    You should have received a copy of the GNU Library General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <stdio.h>
#include <malloc.h>
#include <string.h>

#include <shpell5.h>

#define VECBASE 16

cell make_binary(long val)

  {
  return make(BINARY, val);
  }

cell make_float(double val)

  {
  cell rv;

  rv = lmake(FLOAT);
  value(rv) = val;
  return rv;
  }

cell make_complex(double rep, double imp)

  {
  cell rv;

  rv = lmake(COMPLEX);
  real(rv) = make_float(rep);
  imag(rv) = make_float(imp);
  return rv;
  }

cell make_vector(long l)

  {
  cell rv;
  cell *t;

  rv=givecell();
  id(rv)=VECTOR;

  vecsize(rv)=l;
  vecptr(rv)=(cell *)alloc((long)sizeof(cell)*l);
  t=vecptr(rv);
  while (--l) *t++ = NIL;
  return(rv);
  }

resize_vector(cell v,long l)
 {
 cell *c;
 long newl;

 if (l < vecsize(v))
   return 0;
 newl = ((l >> VECBASE) + 1) << VECBASE ; 
 c=(cell *)alloc((long)sizeof(cell)*newl);
 memset(c,0L,sizeof(cell)*newl);
 memmove(c,vecptr(v),sizeof(cell)*vecsize(v));
 mfree(vecptr(v));
 vecptr(v)=c;
 vecsize(v)=newl;
 }


 
