/*    ILFIVE lisp objects library -- printer
 *
 *    Copyright (c) 1988--1998 Alexandru Dan Corlan M.D.
 * 
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU Library General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    In short, you can use and modify this program. You can
 *    distribute the program with or without your modifications if you want,
 *    but you must not require licence fees and must provide the
 *    full source code with the distribution. If you make an application program
 *    which just calls functions from this library you may distribute that
 *    application program without source code, but also as an linkable object file, 
 *    and must distribute also this library with source code.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU Library General Public License for more details.
 *
 *    You should have received a copy of the GNU Library General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <stdio.h>
#include <string.h>
#include <stdlib.h>

#include <shpell5.h>

int p_esc = 1;
int p_pretty = 1;
int p_circle = 0;
int p_base = 10;
int p_radix = 0;
int p_case = _UPPER_CASE;  /* but ignored here! */
int p_level = 0;
int p_length = 0;
int p_array = 1;
int p_gensym = 1;

static char pbu[256];
static char bb[512];
int tofresh = 1;

extern char *char2str(int c);

pc(char x, ilstream s)

  {
/*
  (is_stream((cell)s) ? (cs_putc(x,stream((cell)s))) : (cs_putc(x,(CSTREAM *)s)));
*/

  ilputch(x,s);
  (x=='\n') ? (tofresh = 1) : (tofresh = 0) ;
  }

cell pstr(char *s, ilstream sr)

  {
  while(*s)
    {
    switch(*s)
      {
/*    case '\n': pc('\\', sr); pc('n', sr); break;
      case '\r': pc('\\', sr); pc('r', sr); break; */
      case '\"': pc('\\', sr); pc('\"', sr); break;
      case '\\': pc('\\', sr); pc('\\', sr); break;
      default: pc(*s, sr); break;
      }
    s++;
    }
  return NIL;
  }

cell pistr(char *s, ilstream sr)

  {
  while(*s) pc(*s++, sr);
  return T;
  }

void ltoa_(char *s, long v)

  {
  sprintf(s, "%ld", v);
  }

void terpri(ilstream sr)
  {
  pc('\n', sr);
  }

void frline(ilstream sr)
  {
  if (!tofresh) terpri(sr);
  }

int pinstep = 2;
int pindent = 0;

cell prin1(cell obj, ilstream sr)

  {
  set_p_esc(1); /* escs enabled */
  if(P_PRETTY)
    prin3(obj,sr);
  else
    prin2(obj,sr);
  return obj;
  }

cell princ(cell obj, ilstream sr)

  {
  set_p_esc(0); /* escs disabled */
  if(P_PRETTY)
    prin3(obj, sr);
  else
    prin2(obj,sr);
  return obj;
  }


/* print the object in an indented format while
 * the escs are given by P_ESC
 */

cell prin3(cell obj, ilstream sr)

  {
  pindent = 0;
  prin0(obj, sr);
  pc('\n', sr);
  return NIL;
  }

/* local function of prin3 */

cell prin0(cell obj, ilstream sr)


  {
  register lp; /* 1 when starting to overindent */

  if (obj==NIL)
    {
    if (P_PRETTY && P_ESC)
      pstr("() ", sr);
    else
      pstr("nil ", sr);
    return T;
    }

  if(is_list(obj))
    {
    pindent += pinstep;
    pc('(', sr);
    lp = 0;
    while(!lp && obj!=NIL && is_list(obj))
      {
      prin0(car(obj), sr);
      pc(' ', sr);
      if(is_list(car(obj))) lp++;
      obj = cdr(obj);
      }
    while(obj!=NIL && is_list(obj))
      {
      pc('\n', sr);
      for(lp=0; lp<pindent; lp++)pc(' ', sr);
      prin0(car(obj), sr);
      pc(' ', sr);
      obj=cdr(obj);
      }
    pindent -= pinstep;
    if(obj==NIL) pc(')', sr);
    else
      {
      pc(' ', sr);
      pc('.', sr);
      prin0(obj, sr);
      pc(')', sr);
      }
    }
  else
    prin2(obj, sr);
  return NIL;
  }


int has_special_chars(char *s)  /* provisory version */

  {
  if(!*s)
    return 1;  /* void symbols to be printed as || */
  for(; *s; s++)
	 if(isspace(*s))
      return 1;
  return 0;
  }

/* print the object without indenting in lists.
 * this function is sensible to P_ESC.
 */

extern long srposition(ilstream);

cell prin2(cell obj, ilstream sr)

  {
  int (* wtr)(cell obj, ilstream cs);
  int ok;
  int hsc;   /* has special chars */
  long srp;

  if (obj==NIL)
    {
    if (P_PRETTY && P_ESC)
      pstr("() ", sr);
    else
      pstr("nil ", sr);
    return T;
    }
  switch (id(obj))
    {
    case BINARY:
      ltoa_(pbu, binary(obj));
      pstr(pbu, sr);
      return T;
    case FLOAT:
      sprintf(pbu, "%f", value(obj));
      pstr(pbu, sr);
      return T;
    case COMPLEX:
      pc('#', sr);
      pc('C', sr);
      pc('(', sr);
      prin2(real(obj), sr);
      pc(' ', sr);
      prin2(imag(obj), sr);
      pc(')', sr);
      return T;
    case STREAM:
      switch(streamid(obj))
        {
        case CHAR_FILE:
          pstr("#<char-file-stream>", sr);
          break;
        case BINARY_FILE:
          pstr("#<binary-file-stream>", sr);
          break;
		  case KEYBOARD:
          pstr("#<keyboard-stream>", sr);
          break;
        case MEMORY:
          pstr("#<string-stream>", sr);
          break;
		  default:
          pstr("#<some-special-stream>", sr);
          break;
        }
      return T;
    case VECTOR:
      if(P_PRETTY)
        pc('\n', sr);
      pc('#', sr);
      pc('(',sr);
      {
      cell *i;
      i=vecptr(obj);
      srp = srposition(sr);
      while (i < &vnth(obj,vecsize(obj)))
       {
       if(P_PRETTY)
         {
         if(srposition(sr)-64>srp)
           {
           pc('\n', sr);
           srp = srposition(sr);
           }
         }
       prin2(*i++,sr);
       pc(' ',sr);
       }
      }
      pc(')',sr);
      return T;
    case CHAR:
      pc('#', sr);
      pc('\\', sr);
      pstr(char2str(obj->charcell.ccode), sr);
      return T;
    case TIME:
      pc('#', sr);
      pc('T', sr);
      pstr(time_unparse(isst(obj)), sr);
      return T;
    case STRING:
    case PURESTRING:
      if (P_ESC)
        {
        pc('\"', sr);
        pstr(string(obj), sr);
        pc('\"', sr);
        return T;
        }
      else pistr(string(obj), sr);
      return T;
    case SYMBOL:
      if(sympack(obj)==_Keyword)
        pc(':', sr);
      else
      if (sympack(obj)!=NIL && _findsym(symname(obj), crtpack)!=obj)
        {
        /* i.e. not accessible in the current package */
        pstr(_pack_name(sympack(obj)), sr);
        pc(':', sr);
        }
      if (!P_ESC)
        pistr(symname(obj), sr);
      else
        {
        if(P_PRETTY && obj==Nil)
          {
          pstr("()", sr);
          return T;
          }
        hsc = 0;
        if(has_special_chars(symname(obj)))
          hsc = 1;
        if(sympack(obj)==NIL && P_GENSYM)
          {
          pstr("#:", sr);
          if(hsc) pc('|', sr);
          pstr(symname(obj), sr);
          if(hsc) pc('|', sr);
          return T;
          }
        if(hsc) pc('|', sr);
        pstr(symname(obj), sr);  /* But take the cares of page 367 */
        if(hsc) pc('|', sr);
        }
      return T;
    case OBJECT:
		if(!is_symbol(typeof(obj)))
		  slex("No symbol is given as an object type.");  /* No symbol as object type */
      ok = locate_objt(typeof(obj));
		if(ok==-1)
		  wtr = (int (*)(cell,cell))NULL;
		else
		  if(P_ESC)
		    wtr = objt[ok].objt_prin1;
		  else
		    wtr = objt[ok].objt_princ;
/*		if(wtr==NULL)
		  slex1s("No princ method is defined for %s objects.",
                    symname(typeof(obj)));   */
	   pc('#', sr);
		pc('[', sr);
		pstr(symname(typeof(obj)), sr);
		pc(' ', sr);
      if(wtr!=(int (*)(cell,cell))NULL)
		  wtr(obj, sr);
      else 
        pc(']', sr);
		return T;
    case LIST:
      pc('(', sr);
      while(obj!=NIL)
        {
        prin2(car(obj), sr);
        pc (' ', sr);
        obj = cdr(obj);
        if (obj==NIL)
          {
          pc(')', sr);
          return T;
          }
        if (id(obj)!=LIST)
          {
          pc('.', sr);
          prin2(obj, sr);
          pc(')', sr);
          return T;
          }
       }
    pc(')', sr);
    return T;
  default:
    slex("Invalid object was given to prin2.");
    return NIL;
  }
}

cell print(cell ce, ilstream sr)

  {
  set_p_pretty(0);
  terpri(sr);
  prin1(ce, sr);
  pc(' ', sr);
  return ce;
  }


cell pprint(cell ce, ilstream sr)

  {
  set_p_pretty(1);
  terpri(sr);
  prin1(ce, sr);
  return ce;
  }


cell pprinc(cell ce, ilstream sr)

  {
  set_p_pretty(1);
  terpri(sr);
  princ(ce, sr);
  return ce;
  }

il5print()

  {
  return 0;
  }

/* End of the Il printer */


