/*    ILFIVE lisp objects library -- reader
 *
 *    Copyright (c) 1988--1998 Alexandru Dan Corlan M.D.
 * 
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU Library General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    In short, you can use and modify this program. You can
 *    distribute the program with or without your modifications if you want,
 *    but you must not require licence fees and must provide the
 *    full source code with the distribution. If you make an application program
 *    which just calls functions from this library you may distribute that
 *    application program without source code, but also as an linkable object file, 
 *    and must distribute also this library with source code.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU Library General Public License for more details.
 *
 *    You should have received a copy of the GNU Library General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <stdio.h>
#include <string.h>
#include <malloc.h>
/* #include <ctype.h>*/

#include <shpell5.h>

#define GEOF EOF

extern long atol();
extern float atof();

/* The following cells shall be all symbols, thus shall not
 * ever be garbage collected.
 */

static cell mread_;
cell eof_obj_;
static cell stdi_;
cell quote_;
cell bq_;
cell co_;

extern cell make_vector(long l);

/* next follows the lread function for reading a Li object in,
 * and the read macros (not user available) that make it work
 */


#define RSIZE 512
#define CQ 0
#define MA 1
#define SE 2
#define ME 3
#define WS 4
#define DE 5
#define CM 6
#define DO 7

extern cell cita_(ilstream);
extern cell hash_(ilstream);
extern cell quoted_(ilstream);
extern cell semico_(ilstream);
extern cell spurious(ilstream);
extern cell openp_(ilstream);
extern cell openb_(ilstream);
extern cell backq_(ilstream);
extern cell comma_(ilstream);

static char rstack[RSIZE];
static char *rtop = rstack;
static int rtable[96] = {
  WS, CQ, MA, MA, CQ, CQ, CQ, MA,  /*  !"'$%&' */
  MA, DE, CQ, CQ, MA, CQ, CQ, CQ,  /* ()*+,-./ */
  CQ, CQ, CQ, CQ, CQ, CQ, CQ, CQ,  /* 01234567 */
  CQ, CQ, CQ, CM, CQ, CQ, CQ, CQ,  /* 89:;<=>? */
  CQ, CQ, CQ, CQ, CQ, CQ, CQ, CQ,  /* @ABCDEFG */
  CQ, CQ, CQ, CQ, CQ, CQ, CQ, CQ,  /* HIJKLMNO */
  CQ, CQ, CQ, CQ, CQ, CQ, CQ, CQ,  /* PQRSTUVW */
  CQ, CQ, CQ, MA, SE, MA, CQ, CQ,  /* XYZ[\]^_ */
  MA, CQ, CQ, CQ, CQ, CQ, CQ, CQ,  /* `abcdefg */
  CQ, CQ, CQ, CQ, CQ, CQ, CQ, CQ,  /* hijklmno */
  CQ, CQ, CQ, CQ, CQ, CQ, CQ, CQ,  /* pqrstuvw */
  CQ, CQ, CQ, CQ, ME, CQ, CQ, CQ   /* xyz{|}~  */
  };

typedef cell (*jumps)();

static jumps rjump[96] =
  {
  0L, 0L, cita_, hash_, 0L, 0L, 0L, quoted_,
  openp_, 0L, 0L, 0L, comma_, 0L, 0L, 0L,
  0L, 0L, 0L, 0L, 0L, 0L, 0L, 0L,
  0L, 0L, 0L, semico_, 0L, 0L, 0L, 0L,
  0L, 0L, 0L, 0L, 0L, 0L, 0L, 0L,
  0L, 0L, 0L, 0L, 0L, 0L, 0L, 0L,
  0L, 0L, 0L, 0L, 0L, 0L, 0L, 0L,
  0L, 0L, 0L, openb_, spurious, 0L, 0L, 0L,
  backq_, 0L, 0L, 0L, 0L, 0L, 0L, 0L,
  0L, 0L, 0L, 0L, 0L, 0L, 0L, 0L,
  0L, 0L, 0L, 0L, 0L, 0L, 0L, 0L,
  0L, 0L, 0L, 0L, spurious, 0L, 0L, 0L
  };

cell spurious(ilstream sr)

  { sr=sr; return NIL; }

cell openb_(ilstream sr)

  { sr=sr; return NIL; }


cell semico_(ilstream sr)

  { sr=sr; return NIL; }

int isbk(char c)

  {
  return (c<=32 && c>=0);
  }

extern cell atommake(char *);

cell lread(ilstream sr)

  {
  char *base, *top;
  int c;

  base = top = rtop = rstack;
  while (isbk(c = ilgetch(sr)));
  if (c==EOF) return eof_obj_;
  ilungetch(c,sr);		/* The first nonblank character */
  if (c==')')
    {
    ilgetch(sr);
    slex("Unexpected ')' was encountered.");
    }
  while (1)
    {
    c = ilgetch(sr);
    if(c==EOF) return eof_obj_;
    if (isbk(c) || rtable[c-32]==DE)
      {
      if (rtable[c-32]==DE) ilungetch(c, sr);
      *top = 0;
      rtop = base;
      return (atommake(base));  /* decimals and symbols */
      }
    if (rtable[c-32]==CQ || ((c & 0xff) > 127) || rjump[c-32]==0L)
      {
      *top++ = c;
      rtop++;
      }
    else switch (rtable[c-32])
      {
      case SE: *top++ = ilgetch(sr);
               rtop++;
               break;
      case ME: while ((c=ilgetch(sr))!='|')
                 {
                 *top++ = c;
                 rtop++;
                 }
               break;
      case CM: while ((c=ilgetch(sr))!='\n');
	       while (isbk(c=ilgetch(sr)));
	       ilungetch(c, sr);
               break;
      default:
               return (*rjump[c-32])(sr);
      }
    }
  }

cell quoted_ (ilstream sr)

  {
  return (cons (quote_, cons( lread(sr), NIL)));
  }

static cell cons_;

cell _backquote(cell form)

  {
  if(form==NIL) return NIL;
  if(!is_list(form))
    return cons(quote_, cons(form, NIL));
  if(car(form)==co_)
    return car(cdr(form));
  return cons(cons_, cons(_backquote(car(form)),
                          cons(_backquote(cdr(form)), NIL)));
  }


cell backq_(ilstream sr)

  {
  return _backquote(lread(sr));
  }

cell comma_(ilstream sr)

  {
  return cons(co_, cons(lread(sr), NIL));
  }

cell openp_(ilstream sr)

  {
  int c;
  register cell locl;

  while (isbk(c = ilgetch(sr)));
  if (c==')') return (NIL);
  if (c=='.')
    {
    locl = lread(sr);
    while (isbk(c = ilgetch(sr)));
    if (c!=')') 
      slex("An unexpected ')' was found.");
    return locl;
    }
  ilungetch(c, sr);                /* different meanings for ']' and others */
  if (c==']') return (NIL);
  locl = lread(sr);
  if (locl==eof_obj_)
    slex("Not all paranthesis were closed before end of file.");
  return cons (locl, openp_(sr));
  }

cell cita_(ilstream sr)

  {
  int c;
  register i;
  register char *top, *base;

  base = top = rtop = rstack;  
  while ((c=ilgetch(sr))!='"')
    {
    if (c=='\\')
      {
      c = ilgetch(sr);
      switch (c)
        {
        case 'n': c = '\n';
                  break;
        case 'r': c = '\r';
                  break;
        case 't': c = '\t';
                  break;
        }
      }
    *top++ = c;
    rtop++;
    }
  rtop = base;
  *top++ = 0;
  i=top-base;
  base = alloc ((long)i);
  strcpy (base, rtop);
  return make (STRING, (long)base);
  }

#define isdigit(c) ((c) >= '0' && (c) <= '9')
#define isupper(c) ((c) >= 'A' && (c) <= 'Z')

#define ischname(c) (c=='+' || isupper(c) || isdigit(c))

cell hash_(ilstream sr)

  {
  int c;
  short i, lo;
  long rv;
  cell rev;
  static char timeitbuf[32];
  cell t;  cell r;
  cell locc;
  cell (* rdr)(ilstream s);

/*  register cell form, rev;*/

  c = ilgetch(sr);
  lo = BINARY;
  switch (c)
    {
    case '\\':
      timeitbuf[0] = ilgetch(sr);
      timeitbuf[1] = ilgetch(sr);
      if(ischname(timeitbuf[1]))
        {
        for(i=1; ischname(timeitbuf[i]); i++)
        	 timeitbuf[i+1] = ilgetch(sr);
        ilungetch(timeitbuf[i], sr);
        timeitbuf[i] = 0;
        timeitbuf[0] = sym2char(esym(timeitbuf));
        }
      else
        ilungetch(timeitbuf[1], sr);
      rev = lmake(CHAR);
      rev->charcell.ccode = (unsigned)(0x00ff & timeitbuf[0]);
      rev->charcell.bits = 0;
      rev->charcell.font = 0;
      return rev;
    case '/':
/*      form = cons( cons(lread(sr), NIL), cons (sr, NIL));   shorthand for a stalk method
      rev = apply (mread_, form);
      ungive (car(form));
      toplout (form);
      return rev;  */
      return NIL;
    case '.':
    case ',':
      /* return eval(lread(sr)); */
      return NIL;
    case 'B':
    case 'b':
      rv = 0;
      while ((c=ilgetch(sr))<='1' && c>='0') rv = rv*2 + (c-'0');
      ilungetch(c, sr);
      return make(lo, rv);
    case 'X':
    case 'x':
      rv = 0;
      while (((c=ilgetch(sr))<='9' && c>='0')
             || (c<='f' && c>='a'))
                rv = rv*16 + (c- (c>='a' ? 'a'-10 : '0'));
      ilungetch(c, sr);
      return make(lo, rv);
    case 'O':
    case 'o':
      rv = 0;
      while ((c=ilgetch(sr))>='0' && c<='7') rv = rv*8 + (c-'0');
      ilungetch(c, sr);
      return make(lo, rv);
    case 'T':
    case 't':
      rev = lmake(TIME);
      i=0;
      do {
         c=ilgetch(sr);
         if(c==' ' || c=='(' || c==')' || c==' ' || c=='\t' || c=='\n' || c=='\r')
           break;
         timeitbuf[i++] = c;
         } while (i<31);
      ilungetch(c, sr);
      timeitbuf[i] = 0;
      time_parse(isst(rev), timeitbuf);
      return rev;
    case '|':
      do {
       	while ((c=ilgetch(sr))!='|' && c!=EOF);
      } while (c!=EOF && (c=ilgetch(sr))!='#');
      return lread(sr);
#ifdef PARSER
    case '`':
      return parse(sr);
#endif
    case '[':
		locc = lread(sr);
		if(!is_symbol(locc))
		  lex(201);	  /* No symbol as object type */
      lo = locate_objt(locc);
		if(lo==-1) 
		  rdr = (cell (*)(cell))NULL;
		else
		  rdr = objt[lo].objt_read;
		if(rdr!=(cell (*)(cell))NULL)
			return (* rdr)(sr);
	   lex(202);   /* No reader is defined */
    case '(':  /* Added by G. Selischi */
      {

      ilungetch(c,sr);
      r=rev=lread(sr);
      t=make_vector(length(rev));
      i=0;
      while(r != NIL)
       {
       vnth(t,i)=car(r);
       r=cdr(r);
       i++;
       }
      toplout(rev);
      return(t);
      }   
    default: break;
    }
  return NIL;
  }

/* observation: there could be other ways in which to implement ';'
   comments, etc., e.g. as a macro that also calls lread(sr); in a more complete
   version, this will certainly be done so.
 */

cell makekwd(char *ss)

  {
  register cell rv;

  rv = emakesym(ss,_Keyword);
  return rv;
  }

cell atommake(char *s)

  {
  char *t;
  short lo;
  static char rpack[64];
  char *pn;
  register cell rv;
/*
  register long r1,r2,r3,r4,r5;
  register char *a1,*a2,*a3;  */
  /* don't know for what did I used these */

/*  printf("atommaking: %s\n", s);*/

  lo = 0;
  t = s;

/*  if (!isdigit(*t)) printf("I don't think |%c| is a digit\n", *t);*/
  if ((*t=='-')||(*t=='+')) t++;
  if (!*t) return (_intern(s, crtpack));  /* it is not exported */

  /* search and work in the event of a float number */
  /* lo == 1 if it is a float */

  lo = 1;
  while(isdigit(*t)) t++;
  if (*t!='.') lo = 0;
  if (!isbk(*t))
    {
    t++;
    while(isdigit(*t))t++;
    if (!(*t=='e' || *t=='E' || isbk(*t))) lo=0;
    if(!isbk(*t))
      {
      t++;
      if(*t=='-' || *t=='+') t++;
      while(isdigit(*t)) t++;
      if(*t && (!isbk(*t))) lo = 0;
      }
    }

  if (lo)
    {
    rv = lmake(FLOAT);
    value(rv) = atof(s);
    return rv;
    }

  lo = 0;

  t = s;
  if ((*t=='-')||(*t=='+')) t++;
  while (*t && isdigit(*t)) if(*t++ == ':') lo++;
  if (*t)
    {
/*    printf("I think it is a symbol %s\n", s);*/
    t = s;
    while (*t) if (*t++ == ':') lo++;
    if (lo!=1) return _intern(s,crtpack);
    if (*s==':') return makekwd(s+1);
    pn = rpack;
    while (*s != ':') *pn++ = *s++;
    *pn = 0;
    s++;
    return pmakesym(s, rpack);
    }
  if (*s=='+')s++;
  else if (*s=='-')
    return make (BINARY, (-atol(s+1)));
  return make (BINARY, (atol(s)));
  }

cell readi(void)

  {
/*
  return reads(car(var(stdi_)));
*/
  return NIL;
  }

cell reads(ilstream sr)

  {
/*
  if (is_string(sr) || is_binary(sr))
    return lread(sr);
*/
  /* the actual exception that prevents the mentioned
   * system crashes is to be raised here
   */
  return NIL;
  }

il5read ()
  {
  mread_ = esym("mread");
  eof_obj_ = esym("end-of-file");
  stdi_ = esym("*standard-in*");
  quote_ = esym("quote");
  bq_ = esym("backquote");
  co_ = esym("comma");
  cons_ = esym("cons");

  return 0;
  }

/* end of reader */

 
