/*    ILFIVE lisp objects library -- system functions
 *
 *    Copyright (c) 1991--1998 Alexandru Dan Corlan M.D.
 * 
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU Library General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    In short, you can use and modify this program. You can
 *    distribute the program with or without your modifications if you want,
 *    but you must not require licence fees and must provide the
 *    full source code with the distribution. If you make an application program
 *    which just calls functions from this library you may distribute that
 *    application program without source code, but also as an linkable object file, 
 *    and must distribute also this library with source code.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU Library General Public License for more details.
 *
 *    You should have received a copy of the GNU Library General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */



#include <ctype.h>
#include <malloc.h>
#include <stdio.h>

#include <shpell5.h>
#include <il5vars.h>

int theread(int n)  /* Takes one argument: the string */

  {
  cell isr;

  if(n==-1)
  	 n = 1;
  if(n!=1 && n!=0)
    slex("Syntax for read is (read [stream | string]).");

  if(n)
    isr = pop();
  else
    isr = csin;

  if(is_string(isr))
    {
    push(slread(string(isr)));
  	 return 1;
    }
  push(lread(isr));
  return 1;
  }

int theprint(int n) 

  {
  cell obj;
  cell isr;

  if(n==-1)
  	 n = 2;
  if(n!=2 && n!=1)
    slex("Syntax for print is (print object [stream | string]).");

  if(n==2)
    isr = pop();
  else
    isr = csout;

  obj = pop();

  if(is_string(isr))
    {
    sprint(string(isr), obj, strsize(isr));
    push(T);
  	 return 1;
    }
  push(print(obj, isr));
  return 1;
  }

int thepprint(int n)  

  {
  cell obj;
  cell isr;

  if(n==-1)
  	 n = 2;
  if(n!=2 && n!=1)
    slex("Syntax for pretty-print is (print object [stream | string]).");

  if(n==2)
    isr = pop();
  else
    isr = csout;

  obj = pop();

  if(is_string(isr))
    {
    spprint(string(isr), obj, strsize(isr));
    push(T);
  	 return 1;
    }
  push(pprint(obj, isr));
  return 1;
  }

/* The format function takes the following arguments:

   - a stream (which could be NIL)
   - a format string
   - a set of items corresponding to items in the string

There are the following formatting items (introduced by ~):

~[-n]A    --- prints with escape characters
~[-n]S    --- just prints the contents of strings and symbols
              and uses print for other situations

n is the number of characters that witll be occupied in the
output; if field is larger, it is TRUNCATED; - means that
allignment should be done to the right

*/

/* size of the format buffer */

#define FFSIZE 1024   

int theformat(int n)

  {
  char *fbuf;
  char *fm;
  char *tt;
  int j, t;
  int fbc; /* format buffer cursor */
  int k;   /* the next argument to be processed */
  cell c;
  int num;
  int minusf;

  if(n<2)
    slex("format expects at least two arguments.");
  fbuf = (char *)malloc(FFSIZE);

  /* n-1 is the stream */
  if(!is_string(*vacc(n-2)))
    {
    mfree(fbuf);
    slex("Second argument of format should be a format string.");
    }
  fm = string(*vacc(n-2));
  k = n-3;
  fbc = 0;
  while(*fm)
    {
    if(*fm!='~')
      {
      fbuf[fbc++] = *fm++;
      continue;
      }
    fm++;
    minusf = 0;
    if(*fm=='-')
      { minusf++; fm++; }
    num = atoi(fm);
    while(isdigit(*fm))
      fm++;
    switch(*fm)
      {
      case 'A':
      case 'S':  /* This happy approach because we shall add others later */
        fm++;
        if(k<0)
    			{
    			mfree(fbuf);
    			slex("Second argument of format should be a format string.");
    			}
        c = *vacc(k);
        if((is_symbol(c) || is_string(c)) && *(fm-1)=='S')
          {
          if(is_symbol(c))
            tt = symname(c);
          else
            tt = string(c);
          if(strlen(tt)+fbc>FFSIZE)
            {
            mfree(fbuf);
            slex("Too long format result.");
            }
          strcpy(fbuf+fbc, tt);
          }
        else
		    sprin1(fbuf+fbc, *vacc(k), (long)FFSIZE-fbc); /* BUT MUST AVOID THE \n */
        k--;
        if(num==0)
          {
          fbc += strlen(fbuf+fbc);
          break;
        	 }
        t = strlen(fbuf+fbc);
        if(t>num)  /* Truncate! */
     		 {
          fbc += num;
          break;
          }
        if(!minusf)
          {
          for(;t<num;t++)
            fbuf[fbc+t] = ' ';
        	 fbc += num;
          break;
          }
        else
          {
			 for(j=fbc+num-1; j>=fbc+num-t; j--)
            fbuf[j] = fbuf[j-(num-t)];
          for(; j>=fbc; j--)
            fbuf[j] = ' ';
        	 fbc += num;
          break;
          }
      }
    }
  fbuf[fbc]=0;
  if(*vacc(n-1)==NIL)
    c = strbuild(fbuf);
  else
    {
    c = *vacc(n-1);
    for(fbc=0; fbuf[fbc]; fbc++)
      ilputch(fbuf[fbc], c);
    c = T;
    }
  for(; n>0; n--)
    drop();
  push(c);
  return 1;  
  }

int theterpri(int n)

  {
  if(n==0)
    ilputch('\n', csout);
  else
    {
    if(!is_stream(*vacc(0)) || n>1)
      slex("A stream expected as single argument to terpri.");
    ilputch('\n', *vacc(0));
    drop();
    }
  push(T);
  return 1;
  }

#ifdef USENAVIGATOR
int theknon(int n)

  {
  cell s;
  cell k;
  cell e;

  if(n==-1)
    n = 3;
  if(n!=3)
    slex("knon syntax: (knon stream key expr).");
  e = pop();
  k = pop();
  if(id(k)!=CHAR)
    slex("Second argument to knon must be a character.");
  s = pop();
  strknon(s, (*k).charcell.ccode, e);
  push(T);
  return 1;
  }

int theknoff(int n)

  {
  cell s;

  if(n==-1)
    n = 1;
  if(n!=1)
    slex("knoff syntax: (knoff stream).");
  s = pop();
  strknoff(s);
  push(T);
  return 1;
  }

int theknedit(int n)

  {
  cell s;
  cell str;

  if(n==-1)
    n = 2;
  if(n!=2)
    slex("knedit syntax: (knedit stream string).");
  str = pop();
  s = pop();
  strknedit(s, string(str), (short)strsize(str) - 1);
  push(T);  /* The contents of the string may be modified
             * by the user during an interaction
             */
  return 1;
  }

int thekninteract(int n)

  {
  cell s;

  if(n==-1)
    n = 1;
  if(n!=1)
    slex("kninteract syntax: (kninteract stream).");
  s = pop();
  s = strkninteract(s);
  push(s);
  return 1;
  }
#endif

/* The string allocation function is written here
 * because of it's relation to the \hlink{knedit} function.
 */

int thestralloc(int n)

  {
  cell s;
  long l;
  char *ss;

  if(n==-1)
    n = 1;
  if(n!=1)
    slex("stralloc has the syntax (stralloc size).");
  s = pop();
  if(!is_binary(s))
    slex("stralloc expects an integer size of the string to be built.");
  ss = malloc((short)(l = binary(s)));
  s = lmake(STRING);
  strsize(s) = (short)l; /* ! */
  string(s) = ss;
  push(s);
  return 1;
  }

/* \section{Seting colours} */

/*

int thesetcolors(int n)

  {
  cell c;

  if(n==-1)
    n = 1;
  if(n!=1)
    slex("set-colors expects one file name argument.");
  c = pop();
  if(!is_string(c))
    slex("Argument to set-colors must be string.");
  if(termlevel==WIN_CHAR)
    set_colors(string(c));
  push(T);
  return 1;
  }

int theeditcolors(int n)

  {
  if(n==-1)
    n = 0;
  if(n)
    slex("edit-colors shpould be called with no argument.");
  edit_colors();
  push(T);
  return 1;
  }
*/

il5sysf()

  {
  bind(esym("format"), makf(theformat));
  bind(esym("pretty-print"), makf(thepprint));
  bind(esym("print"), makf(theprint));
  bind(esym("read"), makf(theread));
  bind(esym("terpri"), makf(theterpri));
#ifdef USENAVIGATOR
  bind(esym("knon"), makf(theknon));
  bind(esym("knoff"), makf(theknoff));
  bind(esym("knedit"), makf(theknedit));
  bind(esym("kninteract"), makf(thekninteract));
#endif
  bind(esym("string-alloc"), makf(thestralloc));
/*
  bind(esym("edit-colors"), makf(theeditcolors));
  bind(esym("set-colors"), makf(thesetcolors));
*/
  }



