--    ILFIVE Ada binding -- Ada part, body
--
--    Copyright (c) 1994--1998 Alexandru Dan Corlan M.D.
-- 
--    This program is free software; you can redistribute it and/or modify
--    it under the terms of the GNU Library General Public License as published by
--    the Free Software Foundation; either version 2 of the License, or
--    (at your option) any later version.
--
--    In short, you can use and modify this program. You can
--    distribute the program with or without your modifications if you want,
--    but you must not require licence fees and must provide the
--    full source code with the distribution. If you make an application program
--    which just calls functions from this library you may distribute that
--    application program without source code, but also as an linkable object file, 
--    and must distribute also this library with source code.
--
--    This program is distributed in the hope that it will be useful,
--    but WITHOUT ANY WARRANTY; without even the implied warranty of
--    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
--    GNU Library General Public License for more details.
--
--    You should have received a copy of the GNU Library General Public License
--    along with this program; if not, write to the Free Software
--    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

with Unchecked_Conversion;
with Text_IO; use Text_IO;

package body Iliada is

Standard_Keyboard: Cell;   

function The_NIL return Cell;
pragma Import(C, The_NIL, "the_nil");

-- conses

function CConsp(C: Cell) return Integer;
pragma Import(C, CConsp, "iliada_consp");

function Consp(C: Cell) return Boolean is

begin
  if CConsp(C)=0 then
    return False;
  else
    return True;
  end if;
end;

function Consp(C: Cell) return Cell is

begin
  if CConsp(C)=0 then
    return NIL;
  else
    return T;
  end if;
end;


function CListp(C: Cell) return Integer;
pragma Import(C, CListp, "iliada_listp");

function Listp(C: Cell) return Boolean is

begin
  if CListp(C)=0 then
    return False;
  else
    return True;
  end if;
end;

function Listp(C: Cell) return Cell is

begin
  if CListp(C)=0 then
    return NIL;
  else
    return T;
  end if;
end;

-- symbols

function mk_sym(sn: char_array) return Cell;
pragma Import(C, mk_sym, "esym");

function Make_Symbol(S: String) return Cell is

begin
  return mk_sym(To_C(S));
end;

function CSymbolp(C: Cell) return Integer;
pragma Import(C, CSymbolp, "iliada_symbolp");

function Symbolp(C: Cell) return Boolean is

begin
  if Csymbolp(C)=0 then
     return False;
  else
     return True;
  end if;
end;


function Symbolp(C: Cell) return Cell is

begin
  if Csymbolp(C)=0 then
     return NIL;
  else
     return T;
  end if;
end;

function symname(C: Cell) return Chars_Ptr;
pragma Import(C, symname, "symbol_name");

function Symbol_Name(C: Cell) return String is

begin
  return Value(symname(C));
end;

-- strings

function strbuild(S: char_array) return Cell;
pragma Import(C, strbuild, "strbuild");

function Make_String(S: String) return Cell is

begin
  return strbuild(To_C(S));
end;


function CStringp(C: Cell) return Integer;
pragma Import(C, CStringp, "iliada_stringp");

function Stringp(C: Cell) return Boolean is

begin
  if Cstringp(C)=0 then
     return False;
  else
     return True;
  end if;
end;


function Stringp(C: Cell) return Cell is

begin
  if Cstringp(C)=0 then
     return NIL;
  else
     return T;
  end if;
end;

function cstringof(C: Cell) return Chars_Ptr;
pragma Import(C, cstringof, "iliada_cstringof");


function String_Of(C: Cell) return String is


begin
  return Value(cstringof(C));
end;


-- sequences

function Length(S: Cell) return Integer is

  R: Integer := 0;
  SS: Cell;

begin
--  if Stringp(S) then
--     return (String_Of(S))'Length;
--  end if;
  SS := S;
  if SS/=NIL and then Consp(SS) then
     while SS /= NIL loop
       R := R + 1;
       SS := Cdr(SS);
       if SS=NIL or else not Consp(SS) then
	  return R;
       end if;
    end loop;
    return R;
  end if;
  raise Sequence_Required;
end Length;

function Nth(N: Integer; S: Cell) return Cell is

  C: Cell := S;

begin
  if Listp(C) then
     for I in 1..N loop
         if C=NIL then
            return NIL;
         end if;
         C := Cdr(C);
     end loop;
     if C = NIL then
        return NIL;
     else
        return Car(C);
     end if;
  else
    raise CL_Type_Error;
  end if;
end Nth;

-- numbers

function CIntegerp(C: Cell) return Integer;
pragma Import(C, CIntegerp, "iliada_integerp");

function Integerp(C: Cell) return Boolean is

begin
  if Cintegerp(C)=0 then
     return False;
  else
     return True;
  end if;
end;


function Integerp(C: Cell) return Cell is

begin
  if Cintegerp(C)=0 then
     return NIL;
  else
     return T;
  end if;
end;


function CFloatp(C: Cell) return Integer;
pragma Import(C, CFloatp, "iliada_floatp");

function Floatp(C: Cell) return Boolean is

begin
  if CFloatp(C)=0 then
     return False;
  else
     return True;
  end if;
end;


function Floatp(C: Cell) return Cell is

begin
  if CFloatp(C)=0 then
     return NIL;
  else
     return T;
  end if;
end;

function Numberp(C: Cell) return Cell is

begin
  if C/=NIL and then (Floatp(C) or Integerp(C)) then
     return T;
  else
     return NIL;
  end if;
end Numberp;

function Numberp(C: Cell) return Boolean is

begin
  if C/=NIL and then (Floatp(C) or Integerp(C)) then
     return True;
  else
     return False;
  end if;
end Numberp;


function CNumberp(C: Cell) return Integer;
pragma Import(C, CNumberp, "iliada_numberp");

--function Numberp(C: Cell) return Boolean is
--
--begin
--  if C/=NIL and then CNumberp(C)=1 then
--     return True;
--  else
--     return False;
--  end if;
--End;


--function Numberp(C: Cell) return Cell is

--begin
--  if  C/=NIL and then CNumberp(C)=0 then
--     return T;
--  else
--     return NIL;
--  end if;
--end;

function The_Integer_Of(C: Cell) return Integer is

begin
  if Integerp(C) then 
     return Integer_Of(C);
  end if;
  if Floatp(C) then 
     return Integer(Float_Of(C));
  end if;
  raise Number_Required;
end The_Integer_Of;

function The_Float_Of(C: Cell) return Float is

begin
  if Integerp(C) then 
     return Float(Integer_Of(C));
  end if;
  if Floatp(C) then
     return Float_Of(C);
  end if;
  raise Number_Required;
end The_Float_Of;




function CStreamp(C: Cell) return Integer;
pragma Import(C, CStreamp, "iliada_streamp");

function Streamp(C: Cell) return Boolean is

begin
  if CStreamp(C)=0 then
     return False;
  else
     return True;
  end if;
end;


function Streamp(C: Cell) return Cell is

begin
  if CStreamp(C)=0 then
     return NIL;
  else
     return T;
  end if;
end;


function The_Standard_In return Cell;
pragma Import(C, The_Standard_In, "iliada_the_csin");

function The_Standard_Out return Cell;
pragma Import(C, The_Standard_Out, "iliada_the_csout");

function The_Standard_Err return Cell;
pragma Import(C, The_Standard_Err, "iliada_the_cserr");

function The_Standard_Kbd return Cell;
pragma Import(C, The_Standard_Kbd, "iliada_the_cskbd");

function ilfopen(FN: char_array; M: short) return Cell;
pragma Import(C, ilfopen, "iliada_fopen");

function Open(File_Name: String; Mode: File_Mode) return Cell is

begin
  case Mode is
  when Input => 
    return ilfopen(To_C(File_Name), short(0));
  when Output =>
    return ilfopen(To_C(File_Name), 1);
  when Input_Output | Append =>
    return ilfopen(To_C(File_Name), 2);
  end case;
end;

procedure il_princ(O, S: Cell);
pragma Import(C, il_princ, "princ");

procedure il_prin1(O, S: Cell);
pragma Import(C, il_prin1, "prin1");

procedure Put(Str, Obj: Cell) is

begin
  il_prin1(Obj, Str);
end Put;

procedure Put(Obj: Cell) is

begin
  il_prin1(Obj, The_Standard_Out);
end;


procedure Princ(Str, Obj: Cell) is

begin
  il_princ(Obj, Str);
end Princ;

procedure Princ(Obj: Cell) is

begin
  il_princ(Obj, The_Standard_Out);
end;


procedure Prin1(Str, Obj: Cell) is

begin
  il_prin1(Obj, Str);
end Prin1;

procedure Prin1(Obj: Cell) is

begin
  il_prin1(Obj, The_Standard_Out);
end;

function lread(S: Cell) return Cell;
pragma Import(C, lread, "lread");

function Read return Cell is

begin
  return lread(Standard_Input);
end Read;

function Read(Str: Cell) return Cell is

begin
  return lread(Str);
end Read;

function Tell_Position(Str: Cell) return Integer;
pragma Import(C, Tell_Position, "srposition");

procedure Set_Position(Str: Cell; P: Integer);
pragma Import(C, Set_Position, "srseek");

function Position(Str: Cell) return Integer is

begin
   return Tell_Position(Str);
end Position;

procedure Position(Str: Cell; P: Integer) is

begin
   Set_Position(Str, P);
end Position;


function Pattern_Check_Low(What: Cell; Pattern: Cell) return Cell;
pragma Import(C, Pattern_Check_Low, "check");

function Pattern_Check(Obj: Cell; Pattern: Cell) return Cell is

begin
  return Pattern_Check_Low(Obj, Pattern);
end Pattern_Check;

function Pattern_Check(Obj: Cell; Pattern: Cell) return Boolean is

  PCRET: Cell;

begin
  PCRET := Pattern_Check_Low(Obj, Pattern);
  if IntegerP(PCRET) then
     return True;
  else 
     return False;
  end if;
end Pattern_Check;

-- Equality

function C_Eql(X, Y: Cell) return Integer;
pragma Import(C, C_Eql, "eql");
function C_Equal(X, Y: Cell) return Integer;
pragma Import(C, C_Equal, "equal");

function Eq(X, Y: Cell) return Boolean is

begin
  if X=Y then
     return True;
  else
     return False;
  end if;
end Eq;


function Eq(X, Y: Cell) return Cell is

begin
  if X=Y then
     return T;
  else
     return NIL;
  end if;
end Eq;

function Eql(X, Y: Cell) return Boolean is

begin
  if C_Eql(X,Y) = 1 then 
     return True;
  else 
     return False;
  end if;
end Eql;


function Eql(X, Y: Cell) return Cell is

begin
  if C_Eql(X,Y) = 1 then 
     return T;
  else 
     return NIL;
  end if;
end Eql;



function Equal(X, Y: Cell) return Boolean is

begin
  if C_Equal(X,Y) = 1 then 
     return True;
  else 
     return False;
  end if;
end Equal;


function Equal(X, Y: Cell) return Cell is

begin
  if C_Equal(X,Y) = 1 then 
     return T;
  else 
     return NIL;
  end if;
end Equal;


-- Sets

function Member(Item, List: Cell;
                Key:        Cell:= NIL;
                Test:       Cell:= Eql_S
                ) return Boolean is

  L: Cell:= List;

begin

  if not Listp(L) then
     raise CL_Type_Error;
  end if;

  while L /= NIL loop      -- Key is ignored, Test can be for the moment
                           -- only Eq_S, Equal_S or Eql_S
    if Test=Eq_S then
       if Car(L)=Item then
          return True;
       end if;
    elsif Test=Eql_S then
       if Eql(Car(L), Item) then
          return True;
       end if;
    elsif Test=Equal_S then
       if Equal(Car(L), Item) then
          return True;
       end if;
    end if;

    L := Cdr(L);
    if not Listp(L) then
       raise CL_Type_Error;       -- it is a dotted list
    end if;
  end loop;

  return False;
  end Member;


function Member(Item, List: Cell;
                Key:        Cell:= NIL;
                Test:       Cell:= Eql_S
                ) return Cell is

begin
  if Member(Item, List, Key, Test) then
     return T;
  else
     return NIL;
  end if;
end;



function Adjoin(Item, List: Cell; 
                Key:        Cell:= NIL;
                Test:       Cell:= Eql_S
                ) return Cell is

begin
  if Member(Item, List, Key, Test) then
     return List;
  else
     return Cons(Item, List);
  end if;
end Adjoin;

-- Initialization

begin
--  Put_Line("Before il5()");
  Il5(0);
--  Put_Line("After il5()");

  NIL := The_NIL;
  T   := Make_Symbol("T");

  Eq_S := Make_Symbol("EQ");
  Eql_S := Make_Symbol("EQL");
  Equal_S := Make_Symbol("EQUAL");

  EOF := Make_Symbol("end-of-file");

  Standard_Output :=   The_Standard_Out;
  Standard_Input :=    The_Standard_In;
  Standard_Error :=    The_Standard_Err;
  Standard_Keyboard := The_Standard_Kbd;

--  Put_Line("Initialization ended");
end Iliada;

