--    ILFIVE Ada binding. Ada part,  header
--
--    Copyright (c) 1994--1998 Alexandru Dan Corlan M.D.
-- 
--    This program is free software; you can redistribute it and/or modify
--    it under the terms of the GNU Library General Public License as published by
--    the Free Software Foundation; either version 2 of the License, or
--    (at your option) any later version.
--
--    In short, you can use and modify this program. You can
--    distribute the program with or without your modifications if you want,
--    but you must not require licence fees and must provide the
--    full source code with the distribution. If you make an application program
--    which just calls functions from this library you may distribute that
--    application program without source code, but also as an linkable object file, 
--    and must distribute also this library with source code.
--
--    This program is distributed in the hope that it will be useful,
--    but WITHOUT ANY WARRANTY; without even the implied warranty of
--    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
--    GNU Library General Public License for more details.
--
--    You should have received a copy of the GNU Library General Public License
--    along with this program; if not, write to the Free Software
--    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.


with Interfaces.C.Strings; 
use Interfaces.C.Strings;

with Interfaces.C;
use Interfaces.C;
with Interfaces.C.Strings;
use Interfaces.C.Strings;

with System;
use System;

with Text_IO; use Text_IO;

package Iliada is

-- What to do about exception management?
-- First let us state what the actual exceptions might be;
-- Then, maybe we select a compilation pragma which
-- implements exceptions---mostly regarding the type
-- of the arguments--with a time penalty.

-- general constants and keywords

type Cell is new Address;

NIL, T: Cell;
EOF: Cell;

Eq_S, Eql_S, Equal_S: Cell;

-- initialisation function

procedure Il5(Mode: short);
pragma Import (C, Il5, "il5");

-- basic management functions

procedure Free(C: Cell);
pragma Import(C, Free, "ungstruct");

-- conses

function Car(C: Cell) return Cell;
pragma Import(C, Car, "iliada_car");
function Cdr(C: Cell) return Cell;
pragma Import(C, Cdr, "iliada_cdr");

function Caar(C: Cell) return Cell;
pragma Import(C, Caar, "iliada_caar");
function Cadr(C: Cell) return Cell;
pragma Import(C, Cadr, "iliada_cadr");
function Cdar(C: Cell) return Cell;
pragma Import(C, Cdar, "iliada_cdar");
function Cddr(C: Cell) return Cell;
pragma Import(C, Cddr, "iliada_cddr");

function Caaar(C: Cell) return Cell;
pragma Import(C, Caaar, "iliada_caaar");
function Caadr(C: Cell) return Cell;
pragma Import(C, Caadr, "iliada_caadr");
function Cadar(C: Cell) return Cell;
pragma Import(C, Cadar, "iliada_cadar");
function Caddr(C: Cell) return Cell;
pragma Import(C, Caddr, "iliada_caddr");
function Cdaar(C: Cell) return Cell;
pragma Import(C, Cdaar, "iliada_cdaar");
function Cdadr(C: Cell) return Cell;
pragma Import(C, Cdadr, "iliada_cdadr");
function Cddar(C: Cell) return Cell;
pragma Import(C, Cddar, "iliada_cddar");
function Cdddr(C: Cell) return Cell;
pragma Import(C, Cdddr, "iliada_cdddr");

function Nth(N: Integer; S: Cell) return Cell;

function Cons(A, B: Cell) return Cell;
pragma Import(C, Cons, "cons");

function Consp(C: Cell) return Boolean;
--function Consp(C: Cell) return Cell;
function Listp(C: Cell) return Boolean;
--function Listp(C: Cell) return Cell;

-- symbols

function Make_Symbol(S: String) return Cell;  -- in the current package
function Symbolp(C: Cell) return Boolean;
--function Symbolp(C: Cell) return Cell;
function Symbol_Name(C: Cell) return String;

-- strings

function Make_String(S: String) return Cell;
function Stringp(C: Cell) return Boolean;
--function Stringp(C: Cell) return Cell;
function String_Of(C: Cell) return String;

-- sequence functions

function Length(S: Cell) return Integer;

-- numbers

function Make_Integer(I: Integer) return Cell;
pragma Import(C, Make_Integer, "iliada_make_int");
function Integerp(C: Cell) return Boolean;
--function Integerp(C: Cell) return Cell;
function Integer_Of(C: Cell) return Integer;
pragma Import(C, Integer_Of, "iliada_int_of");

function Make_Float(F: Float) return Cell;
pragma Import(C, Make_Float, "iliada_make_float");
function Floatp(C: Cell) return Boolean;
--function Floatp(C: Cell) return Cell;
function Float_Of(C: Cell) return Float;
pragma Import(C, Float_Of, "iliada_float_of");

function Numberp(C: Cell) return Boolean;
--function Numberp(C: Cell) return Cell;

   -- aditional functions

function The_Integer_Of(C: Cell) return Integer; -- no matter what numeric
                                                 -- type C has
function The_Float_Of(C: Cell) return Float;     -- idem

-- Input/Output

type File_Mode is (Input, Output, Input_Output, Append);

function Open(File_Name: String; Mode: File_Mode) return Cell;
procedure Close(File: Cell);
pragma Import(C, Close, "iliada_close");

function Streamp(C: Cell) return Boolean;
function Streamp(C: Cell) return Cell;

Standard_Output: Cell;              -- Initialized by the package body
Standard_Input: Cell;
Standard_Error: Cell;

procedure Put(Str, Obj: Cell);      -- Obj on stream Str
procedure Put(Obj: Cell);           -- Obj on stream Standard_Output
--procedure Put(Str: Cell;
--              Obj: String);         -- string Obj on stream Str
--procedure Put(Str: Cell;
--              Obj: Character);      -- character Obj on stream Str

procedure Princ(Str, Obj: Cell);    -- on stream Str
procedure Princ(Obj: Cell);         -- on Standard_Output
--function Princ(Obj: Cell) return String;

procedure Prin1(Str, Obj: Cell);    -- on stream Str
procedure Prin1(Obj: Cell);         -- on Standard_Output
--procedure Prin1(Str: String; Obj: Cell);


function Read return Cell;          -- standard input
function Read(Str: Cell) 
         return Cell;               -- from the given stream
--function Read(Str: String)
--         return Cell;             -- from the given string

function Position(Str: Cell) return Integer;
procedure Position(Str: Cell; P: Integer);

-- The pattern checker

function Pattern_Check(Obj: Cell; Pattern: Cell) return Cell;    -- original
function Pattern_Check(Obj: Cell; Pattern: Cell) return Boolean; -- for Iliada

-- Equality

function Eq(X, Y: Cell)      return Cell;
function Eq(X, Y: Cell)      return Boolean;
function Eql(X, Y: Cell)     return Cell;
function Eql(X, Y: Cell)     return Boolean;
function Equal(X, Y: Cell)   return Cell;
function Equal(X, Y: Cell)   return Boolean;

-- Sets

function Adjoin(Item, List: Cell; 
                Key:        Cell:= NIL;
                Test:       Cell:= Eql_S
                ) return Cell;        -- eventually, a new list

function Member(Item, List: Cell;
                Key:        Cell:= NIL;
                Test:       Cell:= Eql_S
                ) return Boolean;

function Member(Item, List: Cell;
                Key:        Cell:= NIL;
                Test:       Cell:= Eql_S
                ) return Cell;

-- Generally found exceptions

Bad_Cell_Type: exception;           -- a nonspecific exception
Number_Required: exception;         -- Integer or Float Cell required
Sequence_Required: exception;       -- string, list (or vector)

CL_Type_Error: exception;           -- of the Common_Lisp ANSI standard

--private 
--  type Cell is new Address;

end Iliada;

