/*    ILFIVE lisp objects library -- memory management
 *
 *    Copyright (c) 1988--1998 Alexandru Dan Corlan M.D.
 * 
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU Library General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    In short, you can use and modify this program. You can
 *    distribute the program with or without your modifications if you want,
 *    but you must not require licence fees and must provide the
 *    full source code with the distribution. If you make an application program
 *    which just calls functions from this library you may distribute that
 *    application program without source code, but also as an linkable object file, 
 *    and must distribute also this library with source code.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU Library General Public License for more details.
 *
 *    You should have received a copy of the GNU Library General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <stdio.h>
#include <malloc.h>
#include <string.h>

#include <shpell5.h>
#include <il5vars.h>

/* The following functions provide with
 * the implementation of the memory manager
 */

cell freelist;
cell memr[MAXMEMRS];  /* The central register of memory pages */
unsigned maxmemrs = MAXMEMRS;
unsigned actmemrs = 0; /* number of already allocated pages */
cell Nil;
cell t_;

/* The cell allocation system
 */

int addpage(void)

/* This funciton adds a page in the register;
 * if not possible, then it returns 0;
 */

  {
  int i;

  if (actmemrs<maxmemrs)
    {
/*    fprintf(stderr, "Allocating %d.\n", actmemrs);*/
    if ((memr[actmemrs] = alloc(CLUSZ * sizeof(CELL)))!=NULL)
      {
      for (i=0; i<CLUSZ; i++)
        {
        id(&(memr[actmemrs][i])) = LIST;
        car(&(memr[actmemrs][i])) = NIL;
        cdr(&(memr[actmemrs][i])) = freelist;
        freelist = &(memr[actmemrs][i]);
        }
      actmemrs ++;
/*      fprintf(stderr, "Successfully allocated page %d.\n", actmemrs-1);  */
      return 1;  /* O.K. */
      }
    else
      {
      slex1i("There isn't enough memory for a new [%d] cell page.", actmemrs);
      return 0;  /* was not possible */
      }
    }
  slex1i("Allocation of a new [%d] page of memory was not permitted.", actmemrs);
  return 0;  /* was not permitted */
  }

long memstat()  /* prints result on stderr */

  {
  long ncells;
  long nfreecells;
  long nsymbols;
  long nconses;
  long nbinary;
  long nstrings;
  long nfloats;
  long nvectors;
  long ntimes;
  long nother;
  long symbolnames;
  long stringspace;
  long vectorspace;

  int i, j;

  long total;

  ntimes = nfloats = nvectors =
  nsymbols = nconses = nbinary = nstrings = nother = 0;
  vectorspace = symbolnames = stringspace = 0;
  for(i=0; i<actmemrs; i++)
    for(j=0; j<CLUSZ; j++)
    	switch(memr[i][j].conscell.id1)
        {
        case SYMBOL:
          nsymbols ++;
          symbolnames += strlen(memr[i][j].symcell.sname) + 1;
          break;
        case BINARY:
          nbinary ++;
          break;
        case STRING:
          nstrings ++;
          stringspace += strlen(memr[i][j].strcell.sptr) + 1;
          break;
        case LIST:
          nconses ++;
          break;
        case FLOAT:
          nfloats ++;
          break;
        case TIME:
          ntimes ++;
          break;
        case VECTOR:
          nvectors ++;
          vectorspace += memr[i][j].vcell.vsize;
          break;
        default:
          nother ++;
          break;
        }
  fprintf(stderr, "ILFOUR V1.2 --- impure memory allocation status.\n\n");
  fprintf(stderr, "       symbol names occupy ................ %6ld bytes\n",
    symbolnames);
  fprintf(stderr, "       strings occupy ..................... %6ld bytes\n",
    stringspace);
  fprintf(stderr, "       %6ld vector entries .............. %6ld bytes\n",
    vectorspace, 4*vectorspace);
  fprintf(stderr, "       %6ld pages, i.e. %6ld cells..... %6ld bytes\n",
    (long)actmemrs, ncells = CLUSZ * (long)actmemrs, 
    sizeof(CELL) * CLUSZ * (long)actmemrs + 8 * actmemrs);
  fprintf(stderr, "           %6ld cells in the free list.\n",
    nfreecells = (long)length(freelist));
  fprintf(stderr, "           %6ld cons cells.\n", nconses - nfreecells);
  fprintf(stderr, "           %6ld symbol cells.\n", nsymbols);
  fprintf(stderr, "           %6ld string cells.\n", nstrings);
  fprintf(stderr, "           %6ld binary cells.\n", nbinary);
  fprintf(stderr, "           %6ld float cells.\n", nfloats);
  fprintf(stderr, "           %6ld vector cells.\n", nvectors);
  fprintf(stderr, "           %6ld time cells.\n", ntimes);
  fprintf(stderr, "           %6ld other cells.\n\n", nother);
  fprintf(stderr, "Estimated total occupied memory: %6ld bytes.\n",
    total = sizeof(CELL) * CLUSZ * (long)actmemrs + 8 * actmemrs - 
    nfreecells * sizeof(CELL) + 4 * vectorspace + symbolnames +
    stringspace);

  return total;
  }

/* The name allocation subsystem.
 * This utility is used for allocation of strings
 * that are never to be disallocated
 */

static char *crt_page = NULL;
static long chsleft = 0L;

char *name (str)

  register char *str;

  {
  register sl;
  register char *oldp;

  sl = strlen (str) + 1;
  if (sl%2)sl++;
  if (chsleft<sl)
    {
    crt_page = alloc ((long)NAME_PAGE_SIZE);
    chsleft = NAME_PAGE_SIZE;
    }
  if (crt_page==NULL)
    slex("Cannot allocate names page.");
  strcpy (crt_page, str);
  chsleft -= sl;
  oldp = crt_page;
  crt_page += sl;
  return (oldp);
  }

char *salloc(sl)

  register sl;

  {
  register char *oldp;

  if (sl%2) sl++;
  if (chsleft<sl)
    {
    crt_page = alloc((long)NAME_PAGE_SIZE);
    chsleft = NAME_PAGE_SIZE;
    }
  chsleft -= sl;
  oldp = crt_page;
  crt_page += sl;
  return (oldp);
  }

/* shpell(a), the module initialisation function;
 * if the argument is non-zero it says that IL was initiated.
 */

void shpell(int a)

  {
  register i;
  long n;

  for(i=0;i<MAXMEMRS;i++)
    memr[i] = NULL;
  if (addpage()==0)
    {
    printf("Didn't begin\n");
    slex("The first memory page cannot be allocated.");
    exit(0);
    }
  if (a)
    {
    fprintf(stderr, "ILFIVE PORTABLE LISP ENGINE V2.03\n\n");
    }
  }


/* End of the low level memory allocation package */

extern gc();
extern lex(int);

cell givecell ()

  {
  register cell rv;

  if (freelist == NIL)
    {
    if(addpage()==0)
      {
      slex("Cell memory is full and gc isn't implemented yet.");
      exit(-3);
      }
    }
  rv = freelist;
  freelist = cdr(freelist);
  return rv;
  }

cell ungcell(c)

  cell c;

  {
  if(id(c)==FREECELL)
    return NIL;

  /* this prevents corrupting the free list when
   * people disallocate circular structures
   */

  if (is_symbol(c))
    return NIL;  /* by definition they can't be disallocated */
  cdr(c) = freelist;
  car(c) = NIL;
  id(c) = FREECELL;
  freelist = c;
  return NIL;
  }

cell ungstruct(c)

  cell c;

  {
  register i;


  if(c==NIL)
    return NIL;

  /* this prevents corrupting the free list when
   * people disallocate circular structures
   */

  switch(id(c))
    {
    case LIST:
      ungstruct(car(c));
      ungstruct(cdr(c));
      ungcell(c);
      return NIL;
    case STRING:
      mfree(string(c));
      ungcell(c);
      return NIL;
    case PURESTRING:
      ungcell(c);
      return NIL;
    case VECTOR:
      for (i=0; i<vecsize(c); i++)
        ungstruct(vnth(c,i));
      mfree(vecptr(c));
      ungcell(c);
      return NIL;
    case BINARY:
    case FLOAT:
      ungcell(c);
      return NIL;
    case PACKAGE:
    case SYMBOL:
      return NIL;
    case HASHTABLE:
      for(i=0; i<htsize(c); i++)
        {
        ungstruct(htkey(c,i));
        ungstruct(htval(c,i));
        mfree(httab(c));
        mfree(htobj(c));
        }
      ungcell(c);
      return NIL;
    case FREECELL:
      slex("Disallocation of a FREE CELL.");
      return NIL;
    default:
      slex1i("Diasallocation of an unknown type of cell: %d.", id(c));
      ungcell(c);
      return NIL;
    }
  }

/* initialisation values for safer
 * error events
 */

cell _null_vector[2] = {(cell)0, NIL};
cell nofunction() { return NIL; }

cell lmake(t)

  int t;  /* type of the cell */

  {
  register cell rv;

  rv = givecell();
  if (rv==NIL) return rv;
  id(rv) = t;
  switch(t)
    {
    case OBJECT:
      typeof(rv) = NIL;
      reprof(rv) = NIL;
      return rv;
    case STREAM:
      streamid(rv) = SINK;
      thestream(rv) = NULL;
      return rv;
    case BINARY:
      binary(rv) = 0;
      return rv;
    case STRING:
    case PURESTRING:
      string(rv) = "Null";
      return rv;
    case PACKAGE:
      packname(rv) = NIL;
      packstrc(rv) = NULL;
      return rv;
    case HASHTABLE:
      matchtype(rv) = NIL;
      htobj(rv) = NULL;
      return rv;
    case TIME:
      clrtime(isst(rv));
      return rv;
    case CODE:
      codetype(rv) = FUNCTION;
      codefun(rv) = (int (*)(int))NULL;
      return rv;
    case FLOAT:
      value(rv) = 0.0;
      return rv;
    case COMPLEX:
      real(rv) = lmake(FLOAT);
      imag(rv) = lmake(FLOAT);
      return rv;
    case CHAR:
      rv->charcell.ccode = 0;
      rv->charcell.bits = 0;
      rv->charcell.font = 0;
      return rv;
    default:
      slex1i("lmake does not know the given [%d] type.", t);
      return NIL;
    }
  }

/*
static gmrks (c)

  cell c;

  {
  register int *nn;
  register int nx;
  register cell *nc;

  if (is_mrked(c))return NIL;
  if (is_data(c))
    {
    mark(c);
    nn = (int *)(string(c));
    nx = *(nn-1);
    nc = (cell *)(string(c));
    while (nx--)
      gmrks(*nc++);
    return NIL;
    }
  switch (id(c) & UNMASK)
    {
    case SYMBOL:
      mark(c);
      gmrks(var(c));
      gmrks(fun(c));
      gmrks(plist(c));
      break;
    case CODE:
    case BINARY:
    case STRING:
      mark(c);
      break;
    case LIST:
    case OBJECT:
    case FLOAT:
      mark(c);
      gmrks(car(c));
      gmrks(cdr(c));
      break;
    }
  return NIL;
  }
*/

/*
Follow two very old (Il V20.00) garbage collector functions, which shall
not be used any more in the new, C engine runing version.

gc()

  {
  register i, j;
  register n;
  register cell gcrv;

  lowputc (13, cdr(car(var(stdi_))));
  lowputc (10, cdr(car(var(stdi_))));
  lowputc ('G', cdr(car(var(stdi_))));
  n = 0;
  j = gcstop;
  freelist = NIL;
  for (i=0; i<j; i++)
    gmrks (gcs[i]);

  for (i=0; i<sym_size; i++)
    gmrks (i);

  gcstop = j;
  freelist = NIL;

  for (i=0; i<sym_size; i++)
    unmrk(i);
  lowputc ('C', cdr(car(var(stdi_))));

  for (i=sym_size; i<sym_size+mem_size; i++)
    {
    if (!is_mrked(i))
      {
      if (!is_pure(i))
        {
        if (is_string(i) || is_code(i)) mfree (string(i));
        if (is_data(i)) mfree (string(i)-2);
        }
      cdr(i) = freelist;
      car(i) = NIL;
      id(i) = FREE;
      freelist = i;
      n++;
      }
    unmrk(i);
    }
  lowputc('W', cdr(car(var(stdi_))));
  lowputc(' ', cdr(car(var(stdi_))));
  if (n==0) lex(MEF);
  gcrv = make (BINARY, (long)n);
  pts (gcrv, stdi_);
  lowputc (13, cdr(car(var(stdi_))));
  lowputc (10, cdr(car(var(stdi_))));
  return gcrv;
  }
*/

lex(int n)

  {
  printf("Exception %d.\n", n);
  errhandle();
  return 0;  /* no use */
  }

lexs(int n, char *str)

  {
  printf("Exception %d: %s\n", n, str);
  errhandle();
  }


static char appname[32] = "ILFOUR: ";
static ilstream pfcs = NULL;
static char *profn = NULL;

def_app_name(char *name)

  {
  strncpy(appname, name, 31);
  appname[31] = 0;
  }

set_program_file(ilstream cs)

  {
  pfcs = cs;
  }

set_program_name(char *pfn)

  {
  profn = pfn;
  }

static char errb[128];

slex(char *err)

  {
  int i;

  errb[0] = 0;
  if(pfcs!=NULL)
    {
    if(profn==NULL)
      sprintf(errb+strlen(errb), "(%d) ", cs_line_nb(pfcs));
    else
      sprintf(errb+strlen(errb), "(%s %d) ", profn, cs_line_nb(pfcs));
    }
  sprintf(errb+strlen(errb), "%s", appname);
  sprintf(errb+strlen(errb), "%s", err);
  for(i=0; errb[i]; i++)
    ilputch(errb[i], cserr);
  ilputch('\n', cserr);
  errhandle();
  }

slex1s(char *err, char *arg)

  {
  int i;
  errb[0] = 0;
  if(pfcs!=NULL)
    {
    if(profn==NULL)
      sprintf(errb+strlen(errb), "(%d) ", cs_line_nb(pfcs));
    else
      sprintf(errb+strlen(errb), "(%s %d) ", profn, cs_line_nb(pfcs));
    }
  sprintf(errb+strlen(errb), "%s", appname);
  sprintf(errb+strlen(errb), err, arg);  /* err must contain a %s */
  for(i=0; errb[i]; i++)
    ilputch(errb[i], cserr);
  ilputch('\n', cserr);
  errhandle();
  }

slex1i(char *err, unsigned arg)

  {
  int i;
  errb[0] = 0;
  if(pfcs!=NULL)
    {
    if(profn==NULL)
      sprintf(errb+strlen(errb), "(%d) ", cs_line_nb(pfcs));
    else
      sprintf(errb+strlen(errb), "(%s %d) ", profn, cs_line_nb(pfcs));
    }
  sprintf(errb+strlen(errb), "%s", appname);
  sprintf(errb+strlen(errb), err, arg);  /* err must contain a %s */
  for(i=0; errb[i]; i++)
    ilputch(errb[i], cserr);
  ilputch('\n', cserr);
  errhandle();
  }

swarn(char *err)

  {
  int i;

  errb[0] = 0;
  if(pfcs!=NULL)
    {
    if(profn==NULL)
      sprintf(errb+strlen(errb), "(%d) ", cs_line_nb(pfcs));
    else
      sprintf(errb+strlen(errb), "(%s %d) ", profn, cs_line_nb(pfcs));
    }
  sprintf(errb+strlen(errb), "%s ", appname);
  sprintf(errb+strlen(errb), "%s\n", err);
  for(i=0; errb[i]; i++)
    ilputch(errb[i], cserr);
  }

cell pstrbuild(char *str)

  {
  cell rv;

  rv = lmake(STRING);
  str = name(str);
  string(rv) = str;
  strsize(rv) = (long)strlen(str);
  return rv;
  }


cell strbuild(char *str)

  {
  cell rv;
  char * temp;

  rv = lmake(STRING);
  temp = (char *)alloc((long)(strlen(str)+2));
  strcpy(temp,str);
  string(rv) = temp;
  strsize(rv) = (long)strlen(str);
  return rv;
  }


cell make(int idem, long val)

  {
  cell rv;

  rv = lmake(idem);
  switch(idem)
    {
    case BINARY:
      binary(rv) = val;
      return rv;
    case STRING:
    case PURESTRING:
      string(rv) = (char *)val;
      return rv;
    default:
      /* in fact it is an error */
      return NIL;
    }
  }

cell makefloat(double f)

{
  cell r;
  r = lmake(FLOAT);
  value(r) = f;
  return r;
}

cell cons(cell cr, cell cd)

  {
  register cell rv;

  rv = givecell();
  id(rv) = LIST;
  car(rv) = cr;
  cdr(rv) = cd;
  return rv;
  }

/* allocation as for Microsoft C V6.0 */
/*
void *alloc(long nb)

  {
  void *rv;

  rv = malloc((short)nb);
  if(rv==NULL)
    slex("Call to malloc failed.");
  return rv;
  }

void mfree(void *ptr)

  {
  free(ptr);
  }
*/

/* Allocation as for GNU CC */

void *alloc(long nb)

  {
  void *rv;

  rv = malloc((int)nb);
  if(rv==NULL)
    slex("Call to malloc failed.");
  return rv;
  }

void mfree(void *ptr)

  {
  free(ptr);
  }


/* End of the Cells management package */


 
