/*    ILFIVE lisp objects library -- main heading
 *
 *    Copyright (c) 1988--1998 Alexandru Dan Corlan M.D.
 * 
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU Library General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    In short, you can use and modify this program. You can
 *    distribute the program with or without your modifications if you want,
 *    but you must not require licence fees and must provide the
 *    full source code with the distribution. If you make an application program
 *    which just calls functions from this library you may distribute that
 *    application program without source code, but also as an linkable object file, 
 *    and must distribute also this library with source code.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU Library General Public License for more details.
 *
 *    You should have received a copy of the GNU Library General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

/*
\title{ILFIVE - main heading}
\author{Alexandru Dan CORLAN}

\begin{abstract}
This file is the head of the ILFOUR system, containing the
external definitions of the core (i.e. the \hlink{ILFOUR library}
itself).
It should be included by all the ILFOUR applications
immediately after the C headings.
\end{abstract}

\section{Global constants} 

There are three classes of global constants: describing the 
type of ILFOUR that is to be generated by the compiler
(for example having parser or not --- i.e. vizible
from the reader); giving dimensions various structures;
naming binary constants. Such constants also exist
for some other ILFOUR modules but are not included in this
section.

\subsection{conditional compilation}

The following macros are implementation macros that
describe the nature of the machine or of the C
compiler on which this subsystem is
transported

*/

#define ANSIC

/* tells that the machine runs Ansi C */
/*   \subject{ANSIC}  \subject{UNIX7C}  */

#ifndef ANSIC
#define UNIX7C
#endif

/* tells that otherwise Unix V7 C is considered */

#define HASFLOAT

/* tells weather the compiler has a floating-point
 * library  \subject{HASFLOAT} 
 */

/* The following assumed presence of the INDUSS library */
/* It isn't now of any more use at only isstime survived from that */

#define INDUSS

#define CURSES

/*   \subsection{general named integers} */

#define FREECELL 0 /* \subject{FREECELL} */

#define SYMBOL 1 /* \subject{SYMBOL} */
#define BINARY 2 /* \subject{BINARY} */
#define LIST 3 /* \subject{LIST} */
#define FLOAT 4 /* \subject{FLOAT} */
#define STRING 5 /* \subject{STRING} */
#define PURESTRING 6 /* \subject{PURESTRING} */
#define PACKAGE 7 /* \subject{PACKAGE} */
#define PATHNAME 8 /* \subject{PATHNAME} */
#define RATIO 9 /* \subject{RATIO} */
#define HASHTABLE 10 /* \subject{HASHTABLE} */
#define READTABLE 11 /* \subject{READTABLE} */
#define ARRAY 12 /* \subject{ARRAY} */
#define CHAR 14 /* \subject{CHAR} */
#define STREAM 15 /* \subject{STREAM} */
#define COMPLEX 16 /* \subject{COMPLEX} */
#define RANDSTATE 17 /* \subject{RANDSTATE} */
#define STRUCTURE 18 /* \subject{STRUCTURE} */
#define CLASS 19 /* \subject{CLASS} */
#define OBJECT 20 /* \subject{OBJECT} */
#define CODE 22 /* \subject{CODE} */
#define TIME 23 /* \subject{TIME} */
#define POINTER 24  /* \subject{POINTER} */
#define PLACE 25 /* \subject{PLACE} */
#define CCODE 26 /* Anima-evaluator code */

/* specialised array implementations */

#define BITARRAY 60 /* \subject{BITARRAY} */
#define BITVECTOR 61 /* \subject{BITVECTOR} */
#define VECTOR 62 /* \subject{VECTOR} */
#define INTARRAY 63 /* \subject{INTARRAY} */
#define FLOATARRAY 64 /* \subject{FLOATARRAY} */
#define COMPARRAY 65 /* \subject{COMPARRAY} */

/*   \subsection{dimensions} */

#define NAME_PAGE_SIZE 2048
#define CLUSZ 2048   /* cells in a cluster */
#define MAXMEMRS 2048  /* maximum number of pages */

/*   \section{The fundamental Lisp types in C} 

\subject{NIL}  \subject{NULL}  \subject{place}  \subject{cell} */

#define NO_CELL NULL
#define NIL NULL
#define cell union _cell *
#define place cell *

/* \subject{UNMASK} */

#define UNMASK 0x003f

/*   \section{Auxiliary data structures} 

This section contains the defintions of data types that complete
the  \hlink{CELL} data structure. They are all ultimately
pointed to by some cells.

\subsection{streams} */

#define LARGEST_OP 6 /* \subject{LARGEST_OP} */
#define USERBUFSIZE /* \subject{USERBUFISZE} 
  This is the size of the buffer to contain the current
  line typed in by the user in a window
  */

  /* this only for the clipper */

/* \subsubsection{operations permitted on the stream} 
 \subject{open mode} */

#define INPUT 1 /* \subject{INPUT} */
#define OUTPUT 2 /* \subject{OUTPUT} */
#define INPUT_OUTPUT 3 /* \subject{INPUT_OUTPUT} */
#define PROBE 0 /* \subject{PROBE} */

/* \subsubsection{file format and slot type} */

/* \subject{file format} */

#define CHAR_FILE 0
#define TeX_FILE 1
#define PS_FILE 2
#define BINARY_FILE 3
#define MEMORY 4
#define SCREEN_WINDOW 5
#define KEYBOARD 6
#define SINK 7
#define NAVIGATOR 8

/*  \paragraph{link streams}  \subject{link stream}  */

#define RO_LINK 8
#define WO_LINK 9
#define RW_LINK 10
#define MO_LINK 11

#define LINK_MASK 0x00f8

/*   \subject{history types} */

#define LINE_EDIT 1
#define HIST_EDIT 2

/* \subsubsection{the stream type itself} */

/*   \subject{in_stream} --- description of an internal C stream */
/*   \subject{CSTREAM} */

typedef struct in_stream
  {
  int crmode;  /* mode in which it was connected / opened */
  int styp;    /*  \hlink{direct i/o slot} identifier  */
  int iotyp;   /* May be RO, WO, RW or PROBE */
  void *exte;  /* external environment; it is a string, a cell
                * a file pointer or a WINDOW pointer */
  long size;   /* only applies to strings */
  long fill;   /* only applies to strings */
  int edtyp;   /* if RO or RW and IS_WINDOW, this item
                * is \hlink{LINE_EDIT} or \hlink{EXPR_EDIT}
                */
  cell hist;   /* history if necessary --- can either be represented
                * as a list of strings or as a list of expressions */
  char *userinbuf;
  int userincrs;
  int ugstack[LARGEST_OP];  
  int ugstop;
  int eoflag;
  unsigned line_nb;
  cell unclipst; /* this is not in use anymore */
  int refmode;   /* 0 for each character, 1 for each line */
  } CSTREAM;

/* \subsubsection{the ilstream type} 
This is needed because it was changed so frequently
in the past so it was choosen to be just called ilstream
\subject{ilstream} 
 */

#define ilstream cell 

/*   \subsection{hash tables} */

#define INIT_HT_SIZE 55 /* \subject{INIT_HT_SIZE} */

struct hashent {
  cell key;
  cell value;
  }; /*   \subject{hashent} */

struct hashtab {
  int size;
  int increment;
  int threshold;
  struct hashent *tab;  /* it's size is size */
  };  /*   \subject{hashtab} */

/*   \subsection{the arbitrary object types} */

typedef struct objt_entry
{
  cell objt_name;
  cell (* objt_new)();    /* The parameter list can be variable
                           * different creator function types must be made for each
 								* type in part
 								*/
  int (* objt_delete)(cell ob);
  cell (* objt_read)(cell cs);
  cell (* objt_parse)(cell cs);
  int (* objt_unparse)(cell ob, cell cs);
  int (* objt_prin1)(cell ob, cell cs);
  int (* objt_princ)(cell ob, cell cs);
  int (* objt_mark)(cell ob);
}OBJT_ENTRY;  /*   \subject{OBJT_ENTRY} */


/*  \subsection{packages} */

struct hent {
  short heflags;
  cell hecont;
  };

/* \subsubsection{masks for hent} */

#define EXTERN_HMASK 0x0001 /* \subject{EXTERN_HMASK} */
#define SHADOW_HMASK 0x0002 /* \subject{SHADOW_HMASK} */
#define NOT_FREE_HMASK 0x0004 /* \subject{NOT_FREE_HMASK} */

/* \subsubsection{macro selectors for the hashtable slots} */
/*   \subject{is_external}  \subject{is_internal} */
/*   \subject{is_shadowing}  \subject{is_free}  */

#define is_external(h) (h.heflags & EXTERN_HMASK)
#define is_internal(h) !is_external(h)
#define is_shdowing(h) (h.heflags & SHADOW_HMASK)
#define is_free(h) (!(h.heflags & NOT_FREE_HMASK))

/*   \subsubsection{the paks type} */

struct paks {
  cell usel;
  unsigned incr, iincr, htsz, thres;
  struct hent *htab;
  };  /*   \subject{paks} */

/* \notion{parameter stack} \subject{context}

*/

typedef struct pstack{
  cell stsym;
  cell stval;
  } PSTEM;  /*   \subject{PSTEM} */

typedef PSTEM *CONTEXT; /*   \subject{CONTEXT} */

/* \notion{time} */

typedef struct isstime {
  short __year;       /* this is expected to be 2 bytes */
  unsigned __mmddhh;  /* also 2 bytes */
  unsigned __minsec;  /* same */
  short __szms;       /* size, zone, miliseccond: 2 bytes */
  } ISSTIME;	/*  \subject{ISSTIME}  */

/* \subsubsection{macros that access time items} */

#define year(t)    ((t)->__year)
#define month(t)   (((t)->__mmddhh >> 10) & 0x000f)
#define day(t)     (((t)->__mmddhh >> 5) & 0x001f)
#define hour(t)    ((t)->__mmddhh & 0x001f)
#define sec(t)     ((t)->__minsec & 0x003f)
#undef min
#define min(t)     (((t)->__minsec >> 6) & 0x003f)
#define sign(t)    ((t)->__szms < 0)
#define zone(t)    (((t)->__szms >> 10) & 0x001f)
#define milisec(t) ((t)->__szms & 0x03ff)

/* \subsubsection{macros that set time items} */

#define set_year(t,v)    (((t)->__year) = (v))
#define set_month(t,v)   ((t)->__mmddhh = (t)->__mmddhh & 0x03ff | ((v) << 10) & 0x3c00)
#define set_day(t,v)     ((t)->__mmddhh = (t)->__mmddhh & 0x3c1f | ((v) << 5) & 0x03e0)
#define set_hour(t,v)    ((t)->__mmddhh = (t)->__mmddhh & 0x3fe0 | (v) & 0x001f)
#define set_min(t,v)     ((t)->__minsec = (t)->__minsec & 0x003f | ((v) << 6) & 0x0fc0)
#define set_sec(t,v)     ((t)->__minsec = (t)->__minsec & 0x0fc0 | (v) & 0x003f)
#define set_sign(t,v)    ((v) ? ((t)->__szms |= 0x8000) : ((t)->__szms &= 0x7fff))
#define set_zone(t,v)    ((t)->__szms = (t)->__szms & 0x83ff | ((v) << 10)  & 0x007c)
#define set_milisec(t,v) ((t)->__szms = (t)->__szms & 0xfc00 | (v) & 0x03ff)

/* \subsubsection{absent values for time items} */


#define NO_YEAR     32767
#define NO_MONTH    15
#define NO_DAY      31
#define NO_HOUR     31
#define NO_MIN      63
#define NO_SEC      63
#define NO_ZONE     31
#define NO_MILISEC  1023

/* \subsubsection{checking presence of time items} */

#define has_year(t)     (year(t) != NO_YEAR)
#define has_month(t)    (month(t) != NO_MONTH)
#define has_day(t)      (day(t) != NO_DAY)
#define has_hour(t)     (hour(t) != NO_HOUR)
#define has_min(t)      (min(t) != NO_MIN)
#define has_sec(t)      (sec(t) != NO_SEC)
#define has_zone(t)     (zone(t) != NO_ZONE)
#define has_milisec(t)  (milisec(t) != NO_MILISEC)
#define has_sign(t)     (sign(t))

/* \subsubsection{special features of time} */

#define is_bisect(t) (has_year(t) && (year(t) % 4 == 0 && year(t) % 100 || year(t) % 400))
#define is_bc(t)     (((t)->__year & 0x8000) != 0)

/* \subsubsection{value verification of time items} */

#define may_be_month(m)   (m>=0 && m<12)
#define may_be_day(d)     (d>=0 && d<31)
#define may_be_hour(h)    (h>=0 && h<24)
#define may_be_min(m)     (m>=0 && m<60)
#define may_be_sec(s)     (s>=0 && s<60)
#define may_be_milisec(m) (m>=0 && m<1000)
#define may_be_zone(m)    (m>=0 && m<24)

/* \subsubsection{output modes for time data} */

#define READABLE 0
#define ROMANIAN 1
#define ENGLISH 2

/* \subsubsection{prototypes of low level time functions} */

char *time_unparse(ISSTIME *t);

/* \section{The cell structure itself} \subjcet{cell} \subjcet{CELL} */

typedef union _cell
  {
  struct {
    short id1;
    cell car_;  /* car */
    cell cdr_;  /* cdr */
    } conscell; /* LISTS */
  struct {
    short id2;
    cell otype; /* type (class) of user-classed object */
    cell orep;  /* representation of user-classed object */
    } objcell;  /* STRUCT objects or CLASS objects */
  struct {
    short id3;
    long ssize; /* maximum size of this string */
    char *sptr; /* pointer to the buffer containing it */
    } strcell;  /* STRINGs */
  struct {
    short id4;
    long vsize; /* maximum size of this vector */
    cell *vptr; /* cells representing this vector */
    } vcell;    /* VECTORs */
  struct {
    short id5;
    long numbr; /* numberer, or integer value */
    long denom; /* denominator, or 1 for integer */
    } ratiocell;/* RATIOs and FIXNUMs i.e. binary objects */
#ifdef HASFLOAT
  struct {
    short id6;
    double val; /* floating point value */
    } flcell;   /* FLONUMs */
#endif
  struct {
    short id7;
    char *sname;/* symbol name */
    cell spack; /* the package to which it points */
    } symcell;  /* SYMBOLs */
  struct {
    short id8;
    cell names;
    struct paks *pakstr;  /* which actually represents the package */
    } pakcell;  /* PACKAGEs */
  struct {
    short id9;
    char *unused2;
    cell *pncomps;  /* six components of the pathname */
    } pathcell;     /* PATHNAMEs */
  struct {
    short id10;
    cell httype;    /* eq, eql or equal */
    struct hashtab *hstr;
    } htcell;       /* HASHTABLEs */
  struct {
    short id12;
    long font;
    short bits;
    short ccode;
    } charcell;
  struct {
    short id13;
    cell imag;      /* an 8-byte real number, as a FLONUM */
    cell real;      /* same */
    } compcell;     /* a complex number */
             /* \subject{code object} */
  struct {
    short id16;
    long ftype;  /* yet */
    int (* fpointer)(int nargs);
    } codecell;
  struct {
    short id17;
    long strid;    /* stream id */
    void *cstr;
    } strmcell;
  struct {
    short id18;
    ISSTIME istime;
    } timecell;
  struct {
    short id19;
	 cell obtyp_;
	 void *obrepr_;
	 } obcell;
  struct {
    short id20;
    cell contobj;   /* container object */
    void *inoptr;
    } ptrcell;
  struct {
    short id21;
    cell cobj;      /* container object */
    cell *inoplace;
    } placell;
  } CELL;


/*
\section{Methods of the cell subtypes}
*/

/* \subsection{The CELL general data structure} */

#define id(x) ((*x).codecell.id16)

/*\subsection{The CODE macros} */

#define SPECIAL_FORM 1 /* \subject{SPECIAL_FORM} */
#define FUNCTION 2 /* \subject{FUNCTION} */
#define MACRO 3 /* \subject{MACRO} */

#define is_code(x) ((id(x) & UNMASK) == CODE)  /* \subject{is_code} */
#define codetype(x) ((*x).codecell.ftype)      /* \subject{codetype} */
#define codefun(x) ((*x).codecell.fpointer)    /* \subject{codefun} */

#define is_function(x) (is_code(x) && codetype(x)==FUNCTION)
                                               /* \subject{is_function} */
#define is_special_form(x) (is_code(x) && codetype(x)==SPECIAL_FORM)
                                               /* \subject{is_special_form} */
#define is_macro(x) (is_code(x) && codetype(x)==MACRO)
                                               /* \subject{is_macro} */

/* \subsection{The LIST macros} */

#define is_list(x) ((id(x) & UNMASK) == LIST || (x)==NIL) /* \subject{is_list} */
#define is_cons(x) ((id(x) & UNMASK) == LIST) /* \subject{is_cons} */
#define car(x) ((*x).conscell.car_) /* \subject{car} */
#define cdr(x) ((*x).conscell.cdr_) /* \subject{cdr} */

/* \subsection{the BINARY macros} */

#define is_binary(x) ((id(x) & UNMASK) == BINARY) /* \subject{is_binary} */
#define binary(x) ((*x).ratiocell.numbr) /* \subject{binary} */

/* \subsection{the STRING macros} */

#define is_string(x) ((id(x) & UNMASK) == STRING || (id(x) & UNMASK) ==PURESTRING) /* \subject{is_string} */
#define string(x) ((*x).strcell.sptr) /* \subject{string} */
#define strsize(x) ((*x).strcell.ssize) /* \subject{strsize} */

/* \subsection{the PACKAGE macros}  \subject{package}  */

#define is_pack(c) ((id(c) & UNMASK) == PACKAGE) /* \subject{is_pack} */
#define packname(x) ((*x).pakcell.names) /* \subject{packname} */
#define packstrc(x) ((*x).pakcell.pakstr) /* \subject{packstrc} */

/* \subsection{the symbol macros}  \subject{symbol}  */

#define is_symbol(x) ((id(x) & UNMASK) == SYMBOL) /* \subject{is_symbol} */
#define symname(x) ((*x).symcell.sname) /* \subject{symname} */
#define sympack(x) ((*x).symcell.spack) /* \subject{sympack} */
#define is_keyword(x) (is_symbol(x) && sympack(x)==_Keyword) /* \subject{is_keyword} */

/* \subsection{the float macros}  \subject{float} */

#define is_float(x) ((id(x) & UNMASK) == FLOAT) /* \subject{is_float} */
#define value(x) ((*x).flcell.val) /* \subject{value} */

/* \subsection{the complex macros} \subject{complex} */

#define is_complex(x) ((id(x) & UNMASK) == COMPLEX) /* \subject{is_complex} */
#define real(x) ((*x).compcell.real) /* \subject{real} */
#define imag(x) ((*x).compcell.imag) /* \subject{imag} */

/* \subsection{the stream macros} \subject{stream} */

#define is_stream(x) ((id(x) & UNMASK) == STREAM) /* \subject{is_stream} */
#define thestream(x) ((*x).strmcell.cstr) /* \subject{stream} */
#define stream(x) ((CSTREAM *)((*x).strmcell.cstr)) /* \subject{stream} */
#define streamcdr(x) cdr((cell)((*x).strmcell.cstr)) /* \subject{streamcdr} */
#define streamcar(x) car((cell)((*x).strmcell.cstr)) /* \subject{streamcar} */
#define streamid(x) ((*x).strmcell.strid)  /* subject{streamid} */
#define is_link_stream(x) (streamid(x) & LINK_MASK) /* \subject{is_link_stream} */
#define is_dio_stream(x) ((streamid(x) & LINKMASK) == 0)  /* \subject{is_dio_stream} */
#define file_of(x) ((FILE *)(stream(x)->exte)) /*   \subject{file_of} */
#define string_of(x) ((char *)(stream(x)->exte)) /*   \subject{string_of} */


/* \subsection{the hashtable macros}  \subject{hashtable} */

#define INCRINCR 30  /* i.e. incr is incremented with 10% of itself */
  /*  \subject{INCRINCR}  */

#define is_hashtab(x) ((id(x) & UNMASK) == HASHTABLE) /* \subject{is_hashtab} */
#define matchtype(x) ((*x).htcell.httype) /* \subject{matchtype} */
#define htobj(x) ((*x).htcell.hstr) /* \subject{htobj} */
#define htsize(x) ((*x).htcell.hstr -> size) /* \subject{htsize} */
#define htincr(x) ((*x).htcell.hstr -> increment) /* \subject{htincr} */
#define htthre(x) ((*x).htcell.hstr -> threshold) /* \subject{htthre} */
#define httab(x) ((*x).htcell.hstr -> tab) /* \subject{httab} */
#define htkey(x,i) ((*x).htcell.hstr -> tab[i].key) /* \subject{htkey} */
#define htval(x,i) ((*x).htcell.hstr -> tab[i].value) /* \subject{htval} */

/* \subsection{the time macros}  \subject{time} */

#define is_time(x) ((id(x) & UNMASK) == TIME) /* \subject{is_time} */
#define isst(x) (&((*x).timecell.istime)) /* \subject{isst} */


#define is_vector(x) ((id(x) & UNMASK)==VECTOR) /* \subject{is_vector} */
#define vecsize(x) ((*x).vcell.vsize) /* \subject{vecsize} */
#define vecptr(x) ((*x).vcell.vptr) /* \subject{vecptr} */
#define vnth(x,n) ((*x).vcell.vptr[n]) /* \subject{vnth} */
#define vnth2(x,y,z) vnth(vnth(x,y),z) /* \subject{vnth2} */
#define vnth3(a,b,c,d) vnth(vnth(vnth(a,b),c),d) /* \subject{vnth3} */

/* \subsection{the arbitrary object macros}  \subject{arbitrary object}  */

#define is_object(x) ((id(x) & UNMASK) == OBJECT) /* \subject{is_object} */
#define typeof(x) ((*x).obcell.obtyp_) /* \subject{typeof} */
#define reprof(x) ((*x).obcell.obrepr_) /* \subject{reprof} */

/*  \subsection{pointer macros} */

#define is_pointer(x) ((id(x) & UNMASK) == POINTER) /*   \subject{is_pointer} */
#define pointedobj(x) ((*x).ptrcell.contobj) /*   \subject{pointedobj} */
#define pointer(x) ((*x).ptrcell.inoptr) /*   \subject{pointer} */

/* \subsection{place macros} */

#define is_place(x) ((id(x) & UNMASK) == PLACE) /* \subject{is_place} */
#define placedobj(x) ((*x).placell.cobj) /*   \subject{placedobj} */
#define placed(x) ((*x).placell.inoplace)  /*   \subject{placed} */
#define referenced(x) (* ((*x).placell.inoplace)) /*   \subject{referenced} */

/*   \section{External definitions and some constants} */

/*   \subsection{regarding arbitrary objects} \subject{arbitrary object}  */

#define MAX_OBJTE 10 /* \subject{MAX_OBJTE} */

extern OBJT_ENTRY objt[MAX_OBJTE];  /*   \subject{objt} \hlink{OBJT_ENTRY}  */
extern int objtsize; /*   \subject{objtsize} */
extern int objk;  /*   \subject{objk} */

/*   \subsection{regarding memory allocation} */

extern void *alloc(long nb);
extern void mfree(void *ptr);

extern cell freelist;
extern unsigned actmemrs, maxmemrs;
extern cell memr[];

/*  \subsection{some global symbols}  */

extern cell Nil;  /*   \subject{Nil} */
extern cell t_;	/*   \subject{t_} */
#define T t_		/*   \subject{T} */

/* \subsection{names defined in il4eq.c}  \subject{il4eq}  */

extern cell eq_;    /*   \subject{eq_} */
extern cell eql_;	  /*   \subject{eql_} */
extern cell equal_; /*   \subject{equal_} */

/* \subsection{ilgod variables and some constants}  \subject{il4god}  */

extern cell AllPacks;  /*  \subject{AllPacks} see il4god */
extern cell crtpack;   /*  \subject{crtpack} current package */

#define _NONE 0 /* \subject{_NONE} */
#define _INTERNAL 1 /* \subject{_INTERNAL} */
#define _EXTERNAL 2 /* \subject{_EXTERNAL} */
#define _INHERITED 3 /* \subject{_INHERITED} */

extern int _2nd_intern;
extern int _2nd_findsym;

/* names of standard symbol packages */

extern cell _Lisp;    /*   \subject{_Lisp} */
extern cell _User;  	 /*   \subject{_User} */
extern cell _System;  /*   \subject{_System} */
extern cell _Keyword; /*   \subject{_Keyword} */

/* \subsection{global variables exported by il4read}  \subject{il4read} */

extern cell quote_;   /*   \subject{quote_} */
extern cell co_;      /*   \subject{co_} \subject{comma}  */
extern cell bq_;      /*   \subject{bq_} \subject{backquote} */
extern cell eof_obj_; /*   \subject{eof_obj_} */

/* \subsection{name of all the keys} */

extern cell keyname[];

/*   \section{prototypes of ILFOUR library functions} */

/* \subsection{Prototype declarations for IL4GOD.C} \subject{il4god}*/

cell _make_pack(char *pn); /* \subject{_make_pack} */
cell _find_pack(char *pn); /* \subject{_find_pack} */
cell _in_pack(char *pn); /* \subject{_in_pack} */
char *_pack_name(cell pk); /* \subject{_pack_name} */
cell pack_nn(cell pk); /* \subject{pack_nn} */
cell _pk_use_l(cell pk); /* \subject{_pk_use_l} */
cell _pk_use_bl(cell pk); /* \subject{_pk_use_bl} */
cell _p_shing_sy(cell pk); /* \subject{_p_shing_sy} */
cell list_all_p(void); /* \subject{list_all_p} */
cell j_makesymbol(char *sn); /* \subject{j_makesymbol} */
int packhash(char *str, int n); /* \subject{packhash} */
cell _findsym(char *str, cell pk); /* \subject{_findsym} */
cell _intern(char *str, cell pk); /* \subject{_intern} */
int _unintern(cell sy, cell pk); /* \subject{_unintern} */
void _exort(cell slors, cell pk); /* \subject{_exort} */
void _unexport(cell slors, cell pk); /* \subject{_unexport} */
void _use(cell pktouse, cell pk); /* \subject{_use} */
void il4packs(void);  /*   \subject{il4packs} */
cell esym(char *snm); /* \subject{esym} */
cell emakesym(char *snm, cell pack); /* \subject{emakesym} */
cell pmakesym(char *,char *); /* \subject{pmakesym} */
void il4god(void);
/* \subsection{Prototype declarations for IL4SETS.C} \subject{il4sets}*/

int memq(cell obj, cell set); /* \subject{memq} */
cell sinsertq(cell obj, cell set); /* \subject{sinsertq} */
cell unionq(cell set1, cell set2); /* \subject{reunionq} */
cell intersectq(cell set1, cell set2); /* \subject{intersectq} */
int is_partial(union _cell *fel);
int is_fel(union _cell *fel);
union _cell *fel_drop(union _cell *fel);
union _cell *fel_minimize(union _cell *fel);
union _cell *fel_shrink(union _cell *fel);
int fel_member(union _cell *what, union _cell *fel);
union _cell *fel_insert(union _cell *fwhat, union _cell *fwhere);
union _cell *fel_jinsert(union _cell *what, union _cell *where, int coef);
union _cell *fel_copy(union _cell *fel);
union _cell *fel_union(union _cell *f1, union _cell *f2);
union _cell *fel_intersect(union _cell *f1, union _cell *f2);
int fel_anytoins(union _cell *l1, union _cell *l2);
long fel_length(union _cell *fel);
long fel_size(union _cell *fel);

/* \subsection{Prototype declarations for SHPELL4.C} \subject{shpell4}*/

int addpage(void); /* \subject{addpage} */
char *name(char *str); /* \subject{name} */
char *salloc(int sl); /* \subject{salloc} */
void shpell(int a); /* \subject{shpell} */
cell givecell(void); /* \subject{givecell} */
cell ungcell(cell c); /* \subject{ungcell} */
cell ungstruct(cell c); /* \subject{ungstruct} */
cell nofunction(void); /* \subject{nofunction} */
cell lmake(int t); /* \subject{lmake} */
int lex(int n); /* \subject{lex} */
cell make(int idem, long val); /* \subject{make} */
cell strbuild(char *); /* \subject{strbuild} */

cell cons(cell, cell); /* \subject{cons} */
long memstat(void);  /* \subject{memstat} */

/* Prototype declarations for IL4IO.C */

union _cell *strconnect(int obj_type, void *obj_description, int open_mode);
void *disconnect(union _cell *sr);
void dcr_mem_size(union _cell *sr, long sz);
union _cell *winopen(int y0, int x0, int yl, int xl, int mode);
union _cell *fileopen(char *filename, int mode, int form);
union _cell *navopen(void);
void strknon(union _cell *sr, int key, union _cell *val);
void strknoff(union _cell *sr);
void strknedit(union _cell *sr, char *str, short strl);
cell strkninteract(union _cell *sr);
int strrefresh(union _cell *sr);
int winbox(union _cell *sr, int attr, int hor);
int getwinch(union _cell *sr);
int srgetch(union _cell *sr);
void srputc(int c, union _cell *sr);
int srungetch(int c, union _cell *sr);
int cs_line_nb(union _cell *sr);
long srposition(union _cell *sr);
void srseek(union _cell *sr, long srp);
union _cell *slread(char *s);
char *sprint(char *s, union _cell *c, long sz);
char *sprin1(char *s, union _cell *c, long sz);
char *spprint(char *s, union _cell *c, long sz);
void gsetattr(union _cell *sr, short attr);
int il4io(void);
int winstreams(void);

/* Prototype declarations for IL4NUMBR.C */

union _cell *make_binary(long val);
union _cell *make_float(double val);
union _cell *make_complex(double rep, double imp);
union _cell *make_vector(long l);
int resize_vector(union _cell *v, long l);

/* \subsection{Prototype declarations for IL4CHAR.C} */

int sym2char(union _cell *s);
union _cell *char2sym(int c);
char *char2str(int c);
int il4char(void);


/* \subsection{Prototype declarations for IL4PRINT.C} \subject{il4print}*/

int pc(char x, ilstream s); /* \subject{pc} */
cell pstr(char *s, ilstream sr); /* \subject{pstr} */
cell pistr(char *s, ilstream sr); /* \subject{pistr} */
void ltoa_(char *s, long v); /* \subject{ltoa_} */
void terpri(ilstream sr); /* \subject{terpri} */
void frline(ilstream sr); /* \subject{frline} */
cell prin1(cell obj, ilstream sr); /*   \subject{prin1} */
cell princ(cell obj, ilstream sr); /* \subject{princ} */
cell prin3(cell obj, ilstream sr); /*   \subject{prin3} */
cell prin0(cell obj, ilstream sr); /*   \subject{prin0} */
cell prin2(cell obj, ilstream sr);/*   \subject{prin2} */
cell print(cell ce, ilstream sr); /* \subject{print} */
cell pprint(cell ce, ilstream sr); /* \subject{pprint} */
cell pprinc(cell ce, ilstream sr); /* \subject{pprinc} */
int il4print(void); /*   \subject{il4print} */

/* \subsection{Prototype declarations for IL4READ.C} \subject{il4read}*/

cell spurious(ilstream sr); /* \subject{spurious} */
cell openb_(ilstream sr); /* \subject{openb_} */
cell semico_(ilstream sr); /* \subject{semico_} */
int isbk(char c); /* \subject{isbk} */
cell lread(ilstream sr); /* \subject{lread} */
cell quoted_(ilstream sr); /* \subject{quoted_} */
cell _backquote(cell form); /* \subject{_backquote} */
cell backq_(ilstream sr); /* \subject{backq_} */
cell comma_(ilstream sr); /* \subject{comma_} */
cell openp_(ilstream sr); /* \subject{openp_} */
cell cita_(ilstream sr); /* \subject{cita_} */
cell hash_(ilstream sr); /* \subject{hash_} */
cell makekwd(char *ss); /* \subject{makekwd} */
cell atommake(char *s); /* \subject{atommake} */
cell readi(void); /* \subject{readi} */
cell reads(ilstream sr); /* \subject{reads} */
int il4read(void); /*   \subject{il4read} */

/* \subsection{Prototype declarations for IL4LISP.C} \subject{il4lisp}*/

cell dreverse(cell list); /* \subject{dreverse} */
cell copy(cell list); /* \subject{copy} */
cell toplout(cell list); /* \subject{toplout} */
cell topcopy(cell list); /* \subject{topcopy} */
cell topasout(cell list); /* \subject{topasout} */
cell assoc(cell sym, cell alist); /* \subject{assoc} */
cell passoc(cell sym, cell palist); /* \subject{passoc} */

cell nth(cell, int); /* \subject{nth} */
cell nthcdr(cell, int); /* \subject{nthcdr} */

/* \subsection{Prototype declarations for IL4HASHT.C} \subject{il4hasht}*/

long lochash(cell obj); /* \subject{lochash} */
long locshash(cell obj); /* \subject{locshash} */
int cellhash(cell obj, int size); /* \subject{cellhash} */
cell makeht(cell test, int sz, int incr, int thr); /* \subject{makeht} */
int jgethash(cell k, cell ht); /* \subject{jgethash} */
cell hashretrieve(cell ht, cell k, cell def); /* \subject{hashretrieve} */
cell rehash(cell ht); /* \subject{rehash} */
cell hashstore(cell ht, cell k, cell v); /* \subject{hashstore} */
cell *gethash(cell ht, cell k); /* \subject{gethash} */
int remhash(cell k, cell ht); /* \subject{remhash} */

/* \subsection{Prototype declarations for IL4EQ.C} \subject{il4eq}*/

int eql(cell x, cell y); /* \subject{eql} */
int equal(cell x, cell y); /* \subject{equal} */
int is_eq(cell mod, cell x, cell y); /* \subject{is_eq} */
int ileq(void); /* \subject{ileq} */


/* \subsection{Prototype declarations for IL4CHECK.C} \subject{il4check} */

union _cell *mdf(union _cell *expr, int mode);
union _cell *mpfind(union _cell *s);
union _cell *advance_pattern(union _cell *c);
union _cell *listcheck(union _cell *what, union _cell *pattern, int partial);
union _cell *vectorcheck(union _cell *what, union _cell *pattern);
union _cell *check(union _cell *what, union _cell *pattern);
union _cell *_check_(union _cell *expr, int mode);
int il4check(void);

/* \subsection{Prototype declarations for KNAVITOR.C}  \subject{knavitor} */

void knslex(char *err);
void knalloc(long lbs, int lts, int kts, int lsz, int csz);
void knputc(char c);
void knputs(char *s);
void knlf(void);
void knff(void);
void knon(int c, union _cell *val);
void knoff(void);
void knedit(char *, int);
void clrscreen(void);
union _cell *kninteract(void);

/* Prototype declarations for IL4LIFE.C */

union _cell *makf(int (*f)(int n));
union _cell *makm(int (*f)(int n));
union _cell *maks(int (*f)(int n));
void push(union _cell *v);
void drop(void);
void bind(union _cell *s, union _cell *v);
union _cell **sacc(int n);
union _cell **vacc(int n);
union _cell *pop(void);
long binpop(void);
union _cell *symval(union _cell *s);
int lowappl(union _cell *f, int n);
union _cell *appl(union _cell *f, int n);
union _cell *leval(union _cell *expr);
int lambda_interpret(union _cell *f, int n);
int il4life(void);

extern CONTEXT stop;

/* \subsection{Prototype declaration for IL4.C} */

void il4(int a);

/* \section{stream getc and ungetc}  \subject{stream}  
 \subject{ilgetch}  \subject{ilungetch} 
*/

#define ilgetch(s) (srgetch(s))
#define ilungetch(c,s) (srungetch(c,s))
#define ilputch(c,s) (srputc(c,s))

/*   \section{The printing conditions} 

This file is the former printcnd.h

 \subsection{the printing variables} 
*/

extern int p_esc ;
extern int p_pretty ;
extern int p_circle ;
extern int p_base ;
extern int p_radix ;
extern int p_case ;  
extern int p_level ;
extern int p_length ;
extern int p_array ;
extern int p_gensym ;

/*
 \subsection{C access macros for printing variables} 
*/

#define P_ESC p_esc   
#define P_PRETTY p_pretty   
#define P_CIRCLE p_circle   
#define P_BASE p_base   
#define P_RADIX p_radix   
#define P_CASE p_case     
#define P_LEVEL p_level   
#define P_LENGTH p_length   
#define P_ARRAY p_array   
#define P_GENSYM p_gensym   

/*
 \subsection{setting printing variables}
*/

#define set_p_esc(x) p_esc = x
#define set_p_pretty(x) p_pretty = x
#define set_p_circle(x) p_circle = x
#define set_p_base(x) p_base = x
#define set_p_radix(x) p_radix = x
#define set_p_case(x) p_case = x
#define set_p_gensym(x) p_gensym = x
#define set_p_level(x) p_level = x
#define set_p_length(x) p_length = x
#define set_p_array(x) p_array = x

/*
  \subsection{printing case style}

*/

#define _UPPER_CASE 1
#define _LOWER_CASE 2
#define _MIXED_CASE 3

/* \section{Supplementary macros}

   This section contains the former il4mac.h header 

 *
 * This program provides most as macros some of the
 * most frequently used CommonLisp macros, for CommonLisp
 * data structures, and also some implementation macros
 * used in MAGICK.
 *
 * This version is also used in the "big" Il, i.e.
 * the CommonLisp compatible IL V40.00 system.
 *

\subsection{cxxxr}

 */

#define cadr(x) car(cdr(x))
#define cdar(x) cdr(car(x))
#define caar(x) car(car(x))
#define cddr(x) cdr(cdr(x))
#define caaar(x) car(car(car(x)))
#define caadr(x) car(car(cdr(x)))
#define cadar(x) car(cdr(car(x)))
#define caddr(x) car(cdr(cdr(x)))
#define cdaar(x) cdr(car(car(x)))
#define cdadr(x) cdr(car(cdr(x)))
#define cddar(x) cdr(cdr(car(x)))
#define cdddr(x) cdr(cdr(cdr(x)))
#define caaaar(x) car(car(car(car(x))))
#define caaadr(x) car(car(car(cdr(x))))
#define caadar(x) car(car(cdr(car(x))))
#define caaddr(x) car(car(cdr(cdr(x))))
#define cadaar(x) car(cdr(car(car(x))))
#define cadadr(x) car(cdr(car(cdr(x))))
#define caddar(x) car(cdr(cdr(car(x))))
#define cadddr(x) car(cdr(cdr(cdr(x))))
#define cdaaar(x) cdr(car(car(car(x))))
#define cdaadr(x) cdr(car(car(cdr(x))))
#define cdadar(x) cdr(car(cdr(car(x))))
#define cdaddr(x) cdr(car(cdr(cdr(x))))
#define cddaar(x) cdr(cdr(car(car(x))))
#define cddadr(x) cdr(cdr(car(cdr(x))))
#define cdddar(x) cdr(cdr(cdr(car(x))))
#define cddddr(x) cdr(cdr(cdr(cdr(x))))

/*  \subsection{nth} */

#define first(x) car(x)
#define seccond(x) car(cdr(x))
#define third(x) car(cdr(cdr(x)))
#define fourth(x) car(cdr(cdr(cdr(x))))
#define fifth(x) car(cdr(cdr(cdr(cdr(x)))))
#define sixth(x) cadr(cddddr(x))
#define seventh(x) caddr(cddddr(x))
#define eighth(x) cadddr(cddddr(x))
#define nineth(x) caddddr(cddddr(x))

/*  \subsection{nthcdr} */

#define firstcdr(x) cdr(x)
#define seccondcdr(x) cdr(cdr(x))
#define thirdcdr(x) cdr(cdr(cdr(x)))
#define fourthcdr(x) cdr(cdr(cdr(cdr(x))))
#define fifthcdr(x) cdr(cdr(cdr(cdr(cdr(x)))))
#define sixthcdr(x) cddr(cddddr(x))
#define seventhcdr(x) cdddr(cddddr(x))
#define eighthcdr(x) cddddr(cddddr(x))
#define ninethcdr(x) cdddddr(cddddr(x))

/*  \subsection{eq} */

#define eq(x,y) ((x)==(y))

/* End of SHPELL.H */

